#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: metadata_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from . import message_pb2 as message__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='metadata_service.proto',
  package='ai_flow',
  syntax='proto3',
  serialized_options=b'\n\020com.aiflow.protoZ\010/ai_flow\210\001\001\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16metadata_service.proto\x12\x07\x61i_flow\x1a\x1egoogle/protobuf/wrappers.proto\x1a\rmessage.proto\x1a\x1cgoogle/api/annotations.proto\"\x17\n\tIdRequest\x12\n\n\x02id\x18\x01 \x01(\x03\"\x1b\n\x0bNameRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"0\n\x0bListRequest\x12\x11\n\tpage_size\x18\x01 \x01(\x03\x12\x0e\n\x06offset\x18\x02 \x01(\x03\"E\n\x0cMetaResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\t\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t\"@\n\x16RegisterDatasetRequest\x12&\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x15.ai_flow.DatasetProto\"B\n\x17RegisterDatasetsRequest\x12\'\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x15.ai_flow.DatasetProto\";\n\x10\x44\x61tasetListProto\x12\'\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x15.ai_flow.DatasetProto\"\xfc\x04\n\x14UpdateDatasetRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x41\n\nproperties\x18\x02 \x03(\x0b\x32-.ai_flow.UpdateDatasetRequest.PropertiesEntry\x12\x11\n\tname_list\x18\x03 \x03(\t\x12)\n\ttype_list\x18\x04 \x03(\x0e\x32\x16.ai_flow.DataTypeProto\x12\x31\n\x0b\x64\x61ta_format\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12)\n\x03uri\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x31\n\x0b\x64\x65scription\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x32\n\x0c\x63\x61talog_name\x18\t \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x32\n\x0c\x63\x61talog_type\x18\n \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x36\n\x10\x63\x61talog_database\x18\x0b \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12<\n\x16\x63\x61talog_connection_uri\x18\x0c \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x33\n\rcatalog_table\x18\r \x01(\x0b\x32\x1c.google.protobuf.StringValue\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"S\n\x1cRegisterModelRelationRequest\x12\x33\n\x0emodel_relation\x18\x01 \x01(\x0b\x32\x1b.ai_flow.ModelRelationProto\":\n\x14RegisterModelRequest\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.ai_flow.ModelProto\"N\n\x16ModelRelationListProto\x12\x34\n\x0fmodel_relations\x18\x01 \x03(\x0b\x32\x1b.ai_flow.ModelRelationProto\"9\n\x17ModelVersionNameRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08model_id\x18\x02 \x01(\x03\"i\n#RegisterModelVersionRelationRequest\x12\x42\n\x16model_version_relation\x18\x01 \x01(\x0b\x32\".ai_flow.ModelVersionRelationProto\"P\n\x1bRegisterModelVersionRequest\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x1a.ai_flow.ModelVersionProto\"V\n\x1fListModelVersionRelationRequest\x12\x10\n\x08model_id\x18\x01 \x01(\x03\x12\x11\n\tpage_size\x18\x02 \x01(\x03\x12\x0e\n\x06offset\x18\x03 \x01(\x03\"[\n\x1dModelVersionRelationListProto\x12:\n\x0emodel_versions\x18\x01 \x03(\x0b\x32\".ai_flow.ModelVersionRelationProto\"K\n\x15ModelVersionListProto\x12\x32\n\x0emodel_versions\x18\x01 \x03(\x0b\x32\x1a.ai_flow.ModelVersionProto\" \n\x10ModelNameRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"@\n\x16RegisterProjectRequest\x12&\n\x07project\x18\x01 \x01(\x0b\x32\x15.ai_flow.ProjectProto\"\xc5\x01\n\x14UpdateProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x41\n\nproperties\x18\x02 \x03(\x0b\x32-.ai_flow.UpdateProjectRequest.PropertiesEntry\x12)\n\x03uri\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\";\n\x10ProjectListProto\x12\'\n\x08projects\x18\x01 \x03(\x0b\x32\x15.ai_flow.ProjectProto\"C\n\x17RegisterArtifactRequest\x12(\n\x08\x61rtifact\x18\x01 \x01(\x0b\x32\x16.ai_flow.ArtifactProto\"\xaf\x02\n\x15UpdateArtifactRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\nproperties\x18\x02 \x03(\x0b\x32..ai_flow.UpdateArtifactRequest.PropertiesEntry\x12\x33\n\rartifact_type\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12)\n\x03uri\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x31\n\x0b\x64\x65scription\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\">\n\x11\x41rtifactListProto\x12)\n\tartifacts\x18\x01 \x03(\x0b\x32\x16.ai_flow.ArtifactProto\"G\n\x17RegisterWorkflowRequest\x12,\n\x08workflow\x18\x01 \x01(\x0b\x32\x1a.ai_flow.WorkflowMetaProto\"\xbb\x01\n\x15UpdateWorkflowRequest\x12\x14\n\x0cproject_name\x18\x01 \x01(\t\x12\x15\n\rworkflow_name\x18\x02 \x01(\t\x12\x42\n\nproperties\x18\x03 \x03(\x0b\x32..ai_flow.UpdateWorkflowRequest.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"B\n\x11WorkflowListProto\x12-\n\tworkflows\x18\x01 \x03(\x0b\x32\x1a.ai_flow.WorkflowMetaProto\"O\n\x14ListWorkflowsRequest\x12\x14\n\x0cproject_name\x18\x01 \x01(\t\x12\x11\n\tpage_size\x18\x02 \x01(\x03\x12\x0e\n\x06offset\x18\x03 \x01(\x03\"B\n\x13WorkflowNameRequest\x12\x14\n\x0cproject_name\x18\x01 \x01(\t\x12\x15\n\rworkflow_name\x18\x02 \x01(\t2\x89\x31\n\x0fMetadataService\x12\x66\n\x0egetDatasetById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"-\x82\xd3\xe4\x93\x02\'\x12%/aiflow/metadata_store/dataset/get/id\x12l\n\x10getDatasetByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"/\x82\xd3\xe4\x93\x02)\x12\'/aiflow/metadata_store/dataset/get/name\x12\x64\n\x0clistDatasets\x12\x14.ai_flow.ListRequest\x1a\x11.ai_flow.Response\"+\x82\xd3\xe4\x93\x02%\x12#/aiflow/metadata_store/dataset/list\x12u\n\x0fregisterDataset\x12\x1f.ai_flow.RegisterDatasetRequest\x1a\x11.ai_flow.Response\".\x82\xd3\xe4\x93\x02(\"#/aiflow/metadata_store/dataset/save:\x01*\x12\x8d\x01\n\x1aregisterDatasetWithCatalog\x12\x1f.ai_flow.RegisterDatasetRequest\x1a\x11.ai_flow.Response\";\x82\xd3\xe4\x93\x02\x35\"0/aiflow/metadata_store/dataset/save_with_catalog:\x01*\x12w\n\x10registerDatasets\x12 .ai_flow.RegisterDatasetsRequest\x1a\x11.ai_flow.Response\".\x82\xd3\xe4\x93\x02(\"#/aiflow/metadata_store/dataset/save:\x01*\x12s\n\rupdateDataset\x12\x1d.ai_flow.UpdateDatasetRequest\x1a\x11.ai_flow.Response\"0\x82\xd3\xe4\x93\x02*\"%/aiflow/metadata_store/dataset/update:\x01*\x12o\n\x11\x64\x65leteDatasetById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"3\x82\xd3\xe4\x93\x02-\"(/aiflow/metadata_store/dataset/delete/id:\x01*\x12u\n\x13\x64\x65leteDatasetByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"5\x82\xd3\xe4\x93\x02/\"*/aiflow/metadata_store/dataset/delete/name:\x01*\x12s\n\x14getModelRelationById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"4\x82\xd3\xe4\x93\x02.\x12,/aiflow/metadata_store/model_relation/get/id\x12y\n\x16getModelRelationByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"6\x82\xd3\xe4\x93\x02\x30\x12./aiflow/metadata_store/model_relation/get/name\x12p\n\x11listModelRelation\x12\x14.ai_flow.ListRequest\x1a\x11.ai_flow.Response\"2\x82\xd3\xe4\x93\x02,\x12*/aiflow/metadata_store/model_relation/list\x12\x88\x01\n\x15registerModelRelation\x12%.ai_flow.RegisterModelRelationRequest\x1a\x11.ai_flow.Response\"5\x82\xd3\xe4\x93\x02/\"*/aiflow/metadata_store/model_relation/save:\x01*\x12|\n\x17\x64\x65leteModelRelationById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\":\x82\xd3\xe4\x93\x02\x34\"//aiflow/metadata_store/model_relation/delete/id:\x01*\x12\x82\x01\n\x19\x64\x65leteModelRelationByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"<\x82\xd3\xe4\x93\x02\x36\"1/aiflow/metadata_store/model_relation/delete/name:\x01*\x12\x62\n\x0cgetModelById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"+\x82\xd3\xe4\x93\x02%\x12#/aiflow/metadata_store/model/get/id\x12h\n\x0egetModelByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"-\x82\xd3\xe4\x93\x02\'\x12%/aiflow/metadata_store/model/get/name\x12o\n\rregisterModel\x12\x1d.ai_flow.RegisterModelRequest\x1a\x11.ai_flow.Response\",\x82\xd3\xe4\x93\x02&\"!/aiflow/metadata_store/model/save:\x01*\x12k\n\x0f\x64\x65leteModelById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"1\x82\xd3\xe4\x93\x02+\"&/aiflow/metadata_store/model/delete/id:\x01*\x12q\n\x11\x64\x65leteModelByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"3\x82\xd3\xe4\x93\x02-\"(/aiflow/metadata_store/model/delete/name:\x01*\x12\x99\x01\n getModelVersionRelationByVersion\x12 .ai_flow.ModelVersionNameRequest\x1a\x11.ai_flow.Response\"@\x82\xd3\xe4\x93\x02:\x12\x38/aiflow/metadata_store/modelVersion_relation/get/version\x12\x92\x01\n\x18listModelVersionRelation\x12(.ai_flow.ListModelVersionRelationRequest\x1a\x11.ai_flow.Response\"9\x82\xd3\xe4\x93\x02\x33\x12\x31/aiflow/metadata_store/modelVersion_relation/list\x12\x9d\x01\n\x1cregisterModelVersionRelation\x12,.ai_flow.RegisterModelVersionRelationRequest\x1a\x11.ai_flow.Response\"<\x82\xd3\xe4\x93\x02\x36\"1/aiflow/metadata_store/modelVersion_relation/save:\x01*\x12\xa2\x01\n#deleteModelVersionRelationByVersion\x12 .ai_flow.ModelVersionNameRequest\x1a\x11.ai_flow.Response\"F\x82\xd3\xe4\x93\x02@\";/aiflow/metadata_store/modelVersion_relation/delete/version:\x01*\x12\x88\x01\n\x18getModelVersionByVersion\x12 .ai_flow.ModelVersionNameRequest\x1a\x11.ai_flow.Response\"7\x82\xd3\xe4\x93\x02\x31\x12//aiflow/metadata_store/modelVersion/get/version\x12\x84\x01\n\x14registerModelVersion\x12$.ai_flow.RegisterModelVersionRequest\x1a\x11.ai_flow.Response\"3\x82\xd3\xe4\x93\x02-\"(/aiflow/metadata_store/modelVersion/save:\x01*\x12\x91\x01\n\x1b\x64\x65leteModelVersionByVersion\x12 .ai_flow.ModelVersionNameRequest\x1a\x11.ai_flow.Response\"=\x82\xd3\xe4\x93\x02\x37\"2/aiflow/metadata_store/modelVersion/delete/version:\x01*\x12\x88\x01\n\x17getDeployedModelVersion\x12\x19.ai_flow.ModelNameRequest\x1a\x11.ai_flow.Response\"?\x82\xd3\xe4\x93\x02\x39\x12\x37/aiflow/metadata_store/modelVersion/get_serving_version\x12\x9e\x01\n\x1egetLatestValidatedModelVersion\x12\x19.ai_flow.ModelNameRequest\x1a\x11.ai_flow.Response\"N\x82\xd3\xe4\x93\x02H\x12\x46/aiflow/metadata_store/modelVersion/get_latest_validated_model_version\x12\x9e\x01\n\x1egetLatestGeneratedModelVersion\x12\x19.ai_flow.ModelNameRequest\x1a\x11.ai_flow.Response\"N\x82\xd3\xe4\x93\x02H\x12\x46/aiflow/metadata_store/modelVersion/get_latest_generated_model_version\x12\x66\n\x0egetProjectById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"-\x82\xd3\xe4\x93\x02\'\x12%/aiflow/metadata_store/project/get/id\x12l\n\x10getProjectByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"/\x82\xd3\xe4\x93\x02)\x12\'/aiflow/metadata_store/project/get/name\x12u\n\x0fregisterProject\x12\x1f.ai_flow.RegisterProjectRequest\x1a\x11.ai_flow.Response\".\x82\xd3\xe4\x93\x02(\"#/aiflow/metadata_store/project/save:\x01*\x12s\n\rupdateProject\x12\x1d.ai_flow.UpdateProjectRequest\x1a\x11.ai_flow.Response\"0\x82\xd3\xe4\x93\x02*\"%/aiflow/metadata_store/project/update:\x01*\x12\x63\n\x0blistProject\x12\x14.ai_flow.ListRequest\x1a\x11.ai_flow.Response\"+\x82\xd3\xe4\x93\x02%\x12#/aiflow/metadata_store/project/list\x12o\n\x11\x64\x65leteProjectById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"3\x82\xd3\xe4\x93\x02-\"(/aiflow/metadata_store/project/delete/id:\x01*\x12u\n\x13\x64\x65leteProjectByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"5\x82\xd3\xe4\x93\x02/\"*/aiflow/metadata_store/project/delete/name:\x01*\x12h\n\x0fgetArtifactById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\".\x82\xd3\xe4\x93\x02(\x12&/aiflow/metadata_store/artifact/get/id\x12n\n\x11getArtifactByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"0\x82\xd3\xe4\x93\x02*\x12(/aiflow/metadata_store/artifact/get/name\x12v\n\x0eupdateArtifact\x12\x1e.ai_flow.UpdateArtifactRequest\x1a\x11.ai_flow.Response\"1\x82\xd3\xe4\x93\x02+\"&/aiflow/metadata_store/artifact/update:\x01*\x12x\n\x10registerArtifact\x12 .ai_flow.RegisterArtifactRequest\x1a\x11.ai_flow.Response\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metadata_store/artifact/save:\x01*\x12\x65\n\x0clistArtifact\x12\x14.ai_flow.ListRequest\x1a\x11.ai_flow.Response\",\x82\xd3\xe4\x93\x02&\x12$/aiflow/metadata_store/artifact/list\x12q\n\x12\x64\x65leteArtifactById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"4\x82\xd3\xe4\x93\x02.\")/aiflow/metadata_store/artifact/delete/id:\x01*\x12w\n\x14\x64\x65leteArtifactByName\x12\x14.ai_flow.NameRequest\x1a\x11.ai_flow.Response\"6\x82\xd3\xe4\x93\x02\x30\"+/aiflow/metadata_store/artifact/delete/name:\x01*\x12x\n\x10registerWorkflow\x12 .ai_flow.RegisterWorkflowRequest\x1a\x11.ai_flow.Response\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metadata_store/workflow/save:\x01*\x12v\n\x0eupdateWorkflow\x12\x1e.ai_flow.UpdateWorkflowRequest\x1a\x11.ai_flow.Response\"1\x82\xd3\xe4\x93\x02+\"&/aiflow/metadata_store/workflow/update:\x01*\x12h\n\x0fgetWorkflowById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\".\x82\xd3\xe4\x93\x02(\x12&/aiflow/metadata_store/workflow/get/id\x12v\n\x11getWorkflowByName\x12\x1c.ai_flow.WorkflowNameRequest\x1a\x11.ai_flow.Response\"0\x82\xd3\xe4\x93\x02*\x12(/aiflow/metadata_store/workflow/get/name\x12q\n\x12\x64\x65leteWorkflowById\x12\x12.ai_flow.IdRequest\x1a\x11.ai_flow.Response\"4\x82\xd3\xe4\x93\x02.\")/aiflow/metadata_store/workflow/delete/id:\x01*\x12\x7f\n\x14\x64\x65leteWorkflowByName\x12\x1c.ai_flow.WorkflowNameRequest\x1a\x11.ai_flow.Response\"6\x82\xd3\xe4\x93\x02\x30\"+/aiflow/metadata_store/workflow/delete/name:\x01*\x12o\n\rlistWorkflows\x12\x1d.ai_flow.ListWorkflowsRequest\x1a\x11.ai_flow.Response\",\x82\xd3\xe4\x93\x02&\x12$/aiflow/metadata_store/workflow/listB\"\n\x10\x63om.aiflow.protoZ\x08/ai_flow\x88\x01\x01\x90\x01\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,message__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])




_IDREQUEST = _descriptor.Descriptor(
  name='IdRequest',
  full_name='ai_flow.IdRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ai_flow.IdRequest.id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=112,
  serialized_end=135,
)


_NAMEREQUEST = _descriptor.Descriptor(
  name='NameRequest',
  full_name='ai_flow.NameRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ai_flow.NameRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=164,
)


_LISTREQUEST = _descriptor.Descriptor(
  name='ListRequest',
  full_name='ai_flow.ListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ai_flow.ListRequest.page_size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='offset', full_name='ai_flow.ListRequest.offset', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=166,
  serialized_end=214,
)


_METARESPONSE = _descriptor.Descriptor(
  name='MetaResponse',
  full_name='ai_flow.MetaResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.MetaResponse.return_code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.MetaResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='ai_flow.MetaResponse.data', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=216,
  serialized_end=285,
)


_REGISTERDATASETREQUEST = _descriptor.Descriptor(
  name='RegisterDatasetRequest',
  full_name='ai_flow.RegisterDatasetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset', full_name='ai_flow.RegisterDatasetRequest.dataset', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=287,
  serialized_end=351,
)


_REGISTERDATASETSREQUEST = _descriptor.Descriptor(
  name='RegisterDatasetsRequest',
  full_name='ai_flow.RegisterDatasetsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datasets', full_name='ai_flow.RegisterDatasetsRequest.datasets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=353,
  serialized_end=419,
)


_DATASETLISTPROTO = _descriptor.Descriptor(
  name='DatasetListProto',
  full_name='ai_flow.DatasetListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datasets', full_name='ai_flow.DatasetListProto.datasets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=421,
  serialized_end=480,
)


_UPDATEDATASETREQUEST_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='ai_flow.UpdateDatasetRequest.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ai_flow.UpdateDatasetRequest.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ai_flow.UpdateDatasetRequest.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1070,
  serialized_end=1119,
)

_UPDATEDATASETREQUEST = _descriptor.Descriptor(
  name='UpdateDatasetRequest',
  full_name='ai_flow.UpdateDatasetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ai_flow.UpdateDatasetRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='ai_flow.UpdateDatasetRequest.properties', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name_list', full_name='ai_flow.UpdateDatasetRequest.name_list', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type_list', full_name='ai_flow.UpdateDatasetRequest.type_list', index=3,
      number=4, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_format', full_name='ai_flow.UpdateDatasetRequest.data_format', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='ai_flow.UpdateDatasetRequest.uri', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='ai_flow.UpdateDatasetRequest.description', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='catalog_name', full_name='ai_flow.UpdateDatasetRequest.catalog_name', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='catalog_type', full_name='ai_flow.UpdateDatasetRequest.catalog_type', index=8,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='catalog_database', full_name='ai_flow.UpdateDatasetRequest.catalog_database', index=9,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='catalog_connection_uri', full_name='ai_flow.UpdateDatasetRequest.catalog_connection_uri', index=10,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='catalog_table', full_name='ai_flow.UpdateDatasetRequest.catalog_table', index=11,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEDATASETREQUEST_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=483,
  serialized_end=1119,
)


_REGISTERMODELRELATIONREQUEST = _descriptor.Descriptor(
  name='RegisterModelRelationRequest',
  full_name='ai_flow.RegisterModelRelationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_relation', full_name='ai_flow.RegisterModelRelationRequest.model_relation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1121,
  serialized_end=1204,
)


_REGISTERMODELREQUEST = _descriptor.Descriptor(
  name='RegisterModelRequest',
  full_name='ai_flow.RegisterModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model', full_name='ai_flow.RegisterModelRequest.model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1206,
  serialized_end=1264,
)


_MODELRELATIONLISTPROTO = _descriptor.Descriptor(
  name='ModelRelationListProto',
  full_name='ai_flow.ModelRelationListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_relations', full_name='ai_flow.ModelRelationListProto.model_relations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1266,
  serialized_end=1344,
)


_MODELVERSIONNAMEREQUEST = _descriptor.Descriptor(
  name='ModelVersionNameRequest',
  full_name='ai_flow.ModelVersionNameRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ai_flow.ModelVersionNameRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_id', full_name='ai_flow.ModelVersionNameRequest.model_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1346,
  serialized_end=1403,
)


_REGISTERMODELVERSIONRELATIONREQUEST = _descriptor.Descriptor(
  name='RegisterModelVersionRelationRequest',
  full_name='ai_flow.RegisterModelVersionRelationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version_relation', full_name='ai_flow.RegisterModelVersionRelationRequest.model_version_relation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1405,
  serialized_end=1510,
)


_REGISTERMODELVERSIONREQUEST = _descriptor.Descriptor(
  name='RegisterModelVersionRequest',
  full_name='ai_flow.RegisterModelVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='ai_flow.RegisterModelVersionRequest.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1512,
  serialized_end=1592,
)


_LISTMODELVERSIONRELATIONREQUEST = _descriptor.Descriptor(
  name='ListModelVersionRelationRequest',
  full_name='ai_flow.ListModelVersionRelationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_id', full_name='ai_flow.ListModelVersionRelationRequest.model_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ai_flow.ListModelVersionRelationRequest.page_size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='offset', full_name='ai_flow.ListModelVersionRelationRequest.offset', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1594,
  serialized_end=1680,
)


_MODELVERSIONRELATIONLISTPROTO = _descriptor.Descriptor(
  name='ModelVersionRelationListProto',
  full_name='ai_flow.ModelVersionRelationListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_versions', full_name='ai_flow.ModelVersionRelationListProto.model_versions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1682,
  serialized_end=1773,
)


_MODELVERSIONLISTPROTO = _descriptor.Descriptor(
  name='ModelVersionListProto',
  full_name='ai_flow.ModelVersionListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_versions', full_name='ai_flow.ModelVersionListProto.model_versions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1775,
  serialized_end=1850,
)


_MODELNAMEREQUEST = _descriptor.Descriptor(
  name='ModelNameRequest',
  full_name='ai_flow.ModelNameRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ai_flow.ModelNameRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1852,
  serialized_end=1884,
)


_REGISTERPROJECTREQUEST = _descriptor.Descriptor(
  name='RegisterProjectRequest',
  full_name='ai_flow.RegisterProjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='ai_flow.RegisterProjectRequest.project', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1886,
  serialized_end=1950,
)


_UPDATEPROJECTREQUEST_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='ai_flow.UpdateProjectRequest.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ai_flow.UpdateProjectRequest.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ai_flow.UpdateProjectRequest.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1070,
  serialized_end=1119,
)

_UPDATEPROJECTREQUEST = _descriptor.Descriptor(
  name='UpdateProjectRequest',
  full_name='ai_flow.UpdateProjectRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ai_flow.UpdateProjectRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='ai_flow.UpdateProjectRequest.properties', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='ai_flow.UpdateProjectRequest.uri', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEPROJECTREQUEST_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1953,
  serialized_end=2150,
)


_PROJECTLISTPROTO = _descriptor.Descriptor(
  name='ProjectListProto',
  full_name='ai_flow.ProjectListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='projects', full_name='ai_flow.ProjectListProto.projects', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2152,
  serialized_end=2211,
)


_REGISTERARTIFACTREQUEST = _descriptor.Descriptor(
  name='RegisterArtifactRequest',
  full_name='ai_flow.RegisterArtifactRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='artifact', full_name='ai_flow.RegisterArtifactRequest.artifact', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2213,
  serialized_end=2280,
)


_UPDATEARTIFACTREQUEST_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='ai_flow.UpdateArtifactRequest.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ai_flow.UpdateArtifactRequest.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ai_flow.UpdateArtifactRequest.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1070,
  serialized_end=1119,
)

_UPDATEARTIFACTREQUEST = _descriptor.Descriptor(
  name='UpdateArtifactRequest',
  full_name='ai_flow.UpdateArtifactRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ai_flow.UpdateArtifactRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='ai_flow.UpdateArtifactRequest.properties', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='artifact_type', full_name='ai_flow.UpdateArtifactRequest.artifact_type', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='ai_flow.UpdateArtifactRequest.uri', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='ai_flow.UpdateArtifactRequest.description', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEARTIFACTREQUEST_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2283,
  serialized_end=2586,
)


_ARTIFACTLISTPROTO = _descriptor.Descriptor(
  name='ArtifactListProto',
  full_name='ai_flow.ArtifactListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='artifacts', full_name='ai_flow.ArtifactListProto.artifacts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2588,
  serialized_end=2650,
)


_REGISTERWORKFLOWREQUEST = _descriptor.Descriptor(
  name='RegisterWorkflowRequest',
  full_name='ai_flow.RegisterWorkflowRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='workflow', full_name='ai_flow.RegisterWorkflowRequest.workflow', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2652,
  serialized_end=2723,
)


_UPDATEWORKFLOWREQUEST_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='ai_flow.UpdateWorkflowRequest.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ai_flow.UpdateWorkflowRequest.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ai_flow.UpdateWorkflowRequest.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1070,
  serialized_end=1119,
)

_UPDATEWORKFLOWREQUEST = _descriptor.Descriptor(
  name='UpdateWorkflowRequest',
  full_name='ai_flow.UpdateWorkflowRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project_name', full_name='ai_flow.UpdateWorkflowRequest.project_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_name', full_name='ai_flow.UpdateWorkflowRequest.workflow_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='ai_flow.UpdateWorkflowRequest.properties', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEWORKFLOWREQUEST_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2726,
  serialized_end=2913,
)


_WORKFLOWLISTPROTO = _descriptor.Descriptor(
  name='WorkflowListProto',
  full_name='ai_flow.WorkflowListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='workflows', full_name='ai_flow.WorkflowListProto.workflows', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2915,
  serialized_end=2981,
)


_LISTWORKFLOWSREQUEST = _descriptor.Descriptor(
  name='ListWorkflowsRequest',
  full_name='ai_flow.ListWorkflowsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project_name', full_name='ai_flow.ListWorkflowsRequest.project_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ai_flow.ListWorkflowsRequest.page_size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='offset', full_name='ai_flow.ListWorkflowsRequest.offset', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2983,
  serialized_end=3062,
)


_WORKFLOWNAMEREQUEST = _descriptor.Descriptor(
  name='WorkflowNameRequest',
  full_name='ai_flow.WorkflowNameRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project_name', full_name='ai_flow.WorkflowNameRequest.project_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_name', full_name='ai_flow.WorkflowNameRequest.workflow_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3064,
  serialized_end=3130,
)

_REGISTERDATASETREQUEST.fields_by_name['dataset'].message_type = message__pb2._DATASETPROTO
_REGISTERDATASETSREQUEST.fields_by_name['datasets'].message_type = message__pb2._DATASETPROTO
_DATASETLISTPROTO.fields_by_name['datasets'].message_type = message__pb2._DATASETPROTO
_UPDATEDATASETREQUEST_PROPERTIESENTRY.containing_type = _UPDATEDATASETREQUEST
_UPDATEDATASETREQUEST.fields_by_name['properties'].message_type = _UPDATEDATASETREQUEST_PROPERTIESENTRY
_UPDATEDATASETREQUEST.fields_by_name['type_list'].enum_type = message__pb2._DATATYPEPROTO
_UPDATEDATASETREQUEST.fields_by_name['data_format'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEDATASETREQUEST.fields_by_name['uri'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEDATASETREQUEST.fields_by_name['description'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEDATASETREQUEST.fields_by_name['catalog_name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEDATASETREQUEST.fields_by_name['catalog_type'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEDATASETREQUEST.fields_by_name['catalog_database'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEDATASETREQUEST.fields_by_name['catalog_connection_uri'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEDATASETREQUEST.fields_by_name['catalog_table'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_REGISTERMODELRELATIONREQUEST.fields_by_name['model_relation'].message_type = message__pb2._MODELRELATIONPROTO
_REGISTERMODELREQUEST.fields_by_name['model'].message_type = message__pb2._MODELPROTO
_MODELRELATIONLISTPROTO.fields_by_name['model_relations'].message_type = message__pb2._MODELRELATIONPROTO
_REGISTERMODELVERSIONRELATIONREQUEST.fields_by_name['model_version_relation'].message_type = message__pb2._MODELVERSIONRELATIONPROTO
_REGISTERMODELVERSIONREQUEST.fields_by_name['model_version'].message_type = message__pb2._MODELVERSIONPROTO
_MODELVERSIONRELATIONLISTPROTO.fields_by_name['model_versions'].message_type = message__pb2._MODELVERSIONRELATIONPROTO
_MODELVERSIONLISTPROTO.fields_by_name['model_versions'].message_type = message__pb2._MODELVERSIONPROTO
_REGISTERPROJECTREQUEST.fields_by_name['project'].message_type = message__pb2._PROJECTPROTO
_UPDATEPROJECTREQUEST_PROPERTIESENTRY.containing_type = _UPDATEPROJECTREQUEST
_UPDATEPROJECTREQUEST.fields_by_name['properties'].message_type = _UPDATEPROJECTREQUEST_PROPERTIESENTRY
_UPDATEPROJECTREQUEST.fields_by_name['uri'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_PROJECTLISTPROTO.fields_by_name['projects'].message_type = message__pb2._PROJECTPROTO
_REGISTERARTIFACTREQUEST.fields_by_name['artifact'].message_type = message__pb2._ARTIFACTPROTO
_UPDATEARTIFACTREQUEST_PROPERTIESENTRY.containing_type = _UPDATEARTIFACTREQUEST
_UPDATEARTIFACTREQUEST.fields_by_name['properties'].message_type = _UPDATEARTIFACTREQUEST_PROPERTIESENTRY
_UPDATEARTIFACTREQUEST.fields_by_name['artifact_type'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEARTIFACTREQUEST.fields_by_name['uri'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_UPDATEARTIFACTREQUEST.fields_by_name['description'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_ARTIFACTLISTPROTO.fields_by_name['artifacts'].message_type = message__pb2._ARTIFACTPROTO
_REGISTERWORKFLOWREQUEST.fields_by_name['workflow'].message_type = message__pb2._WORKFLOWMETAPROTO
_UPDATEWORKFLOWREQUEST_PROPERTIESENTRY.containing_type = _UPDATEWORKFLOWREQUEST
_UPDATEWORKFLOWREQUEST.fields_by_name['properties'].message_type = _UPDATEWORKFLOWREQUEST_PROPERTIESENTRY
_WORKFLOWLISTPROTO.fields_by_name['workflows'].message_type = message__pb2._WORKFLOWMETAPROTO
DESCRIPTOR.message_types_by_name['IdRequest'] = _IDREQUEST
DESCRIPTOR.message_types_by_name['NameRequest'] = _NAMEREQUEST
DESCRIPTOR.message_types_by_name['ListRequest'] = _LISTREQUEST
DESCRIPTOR.message_types_by_name['MetaResponse'] = _METARESPONSE
DESCRIPTOR.message_types_by_name['RegisterDatasetRequest'] = _REGISTERDATASETREQUEST
DESCRIPTOR.message_types_by_name['RegisterDatasetsRequest'] = _REGISTERDATASETSREQUEST
DESCRIPTOR.message_types_by_name['DatasetListProto'] = _DATASETLISTPROTO
DESCRIPTOR.message_types_by_name['UpdateDatasetRequest'] = _UPDATEDATASETREQUEST
DESCRIPTOR.message_types_by_name['RegisterModelRelationRequest'] = _REGISTERMODELRELATIONREQUEST
DESCRIPTOR.message_types_by_name['RegisterModelRequest'] = _REGISTERMODELREQUEST
DESCRIPTOR.message_types_by_name['ModelRelationListProto'] = _MODELRELATIONLISTPROTO
DESCRIPTOR.message_types_by_name['ModelVersionNameRequest'] = _MODELVERSIONNAMEREQUEST
DESCRIPTOR.message_types_by_name['RegisterModelVersionRelationRequest'] = _REGISTERMODELVERSIONRELATIONREQUEST
DESCRIPTOR.message_types_by_name['RegisterModelVersionRequest'] = _REGISTERMODELVERSIONREQUEST
DESCRIPTOR.message_types_by_name['ListModelVersionRelationRequest'] = _LISTMODELVERSIONRELATIONREQUEST
DESCRIPTOR.message_types_by_name['ModelVersionRelationListProto'] = _MODELVERSIONRELATIONLISTPROTO
DESCRIPTOR.message_types_by_name['ModelVersionListProto'] = _MODELVERSIONLISTPROTO
DESCRIPTOR.message_types_by_name['ModelNameRequest'] = _MODELNAMEREQUEST
DESCRIPTOR.message_types_by_name['RegisterProjectRequest'] = _REGISTERPROJECTREQUEST
DESCRIPTOR.message_types_by_name['UpdateProjectRequest'] = _UPDATEPROJECTREQUEST
DESCRIPTOR.message_types_by_name['ProjectListProto'] = _PROJECTLISTPROTO
DESCRIPTOR.message_types_by_name['RegisterArtifactRequest'] = _REGISTERARTIFACTREQUEST
DESCRIPTOR.message_types_by_name['UpdateArtifactRequest'] = _UPDATEARTIFACTREQUEST
DESCRIPTOR.message_types_by_name['ArtifactListProto'] = _ARTIFACTLISTPROTO
DESCRIPTOR.message_types_by_name['RegisterWorkflowRequest'] = _REGISTERWORKFLOWREQUEST
DESCRIPTOR.message_types_by_name['UpdateWorkflowRequest'] = _UPDATEWORKFLOWREQUEST
DESCRIPTOR.message_types_by_name['WorkflowListProto'] = _WORKFLOWLISTPROTO
DESCRIPTOR.message_types_by_name['ListWorkflowsRequest'] = _LISTWORKFLOWSREQUEST
DESCRIPTOR.message_types_by_name['WorkflowNameRequest'] = _WORKFLOWNAMEREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

IdRequest = _reflection.GeneratedProtocolMessageType('IdRequest', (_message.Message,), {
  'DESCRIPTOR' : _IDREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.IdRequest)
  })
_sym_db.RegisterMessage(IdRequest)

NameRequest = _reflection.GeneratedProtocolMessageType('NameRequest', (_message.Message,), {
  'DESCRIPTOR' : _NAMEREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.NameRequest)
  })
_sym_db.RegisterMessage(NameRequest)

ListRequest = _reflection.GeneratedProtocolMessageType('ListRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListRequest)
  })
_sym_db.RegisterMessage(ListRequest)

MetaResponse = _reflection.GeneratedProtocolMessageType('MetaResponse', (_message.Message,), {
  'DESCRIPTOR' : _METARESPONSE,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetaResponse)
  })
_sym_db.RegisterMessage(MetaResponse)

RegisterDatasetRequest = _reflection.GeneratedProtocolMessageType('RegisterDatasetRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERDATASETREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterDatasetRequest)
  })
_sym_db.RegisterMessage(RegisterDatasetRequest)

RegisterDatasetsRequest = _reflection.GeneratedProtocolMessageType('RegisterDatasetsRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERDATASETSREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterDatasetsRequest)
  })
_sym_db.RegisterMessage(RegisterDatasetsRequest)

DatasetListProto = _reflection.GeneratedProtocolMessageType('DatasetListProto', (_message.Message,), {
  'DESCRIPTOR' : _DATASETLISTPROTO,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.DatasetListProto)
  })
_sym_db.RegisterMessage(DatasetListProto)

UpdateDatasetRequest = _reflection.GeneratedProtocolMessageType('UpdateDatasetRequest', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEDATASETREQUEST_PROPERTIESENTRY,
    '__module__' : 'metadata_service_pb2'
    # @@protoc_insertion_point(class_scope:ai_flow.UpdateDatasetRequest.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _UPDATEDATASETREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UpdateDatasetRequest)
  })
_sym_db.RegisterMessage(UpdateDatasetRequest)
_sym_db.RegisterMessage(UpdateDatasetRequest.PropertiesEntry)

RegisterModelRelationRequest = _reflection.GeneratedProtocolMessageType('RegisterModelRelationRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERMODELRELATIONREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterModelRelationRequest)
  })
_sym_db.RegisterMessage(RegisterModelRelationRequest)

RegisterModelRequest = _reflection.GeneratedProtocolMessageType('RegisterModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERMODELREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterModelRequest)
  })
_sym_db.RegisterMessage(RegisterModelRequest)

ModelRelationListProto = _reflection.GeneratedProtocolMessageType('ModelRelationListProto', (_message.Message,), {
  'DESCRIPTOR' : _MODELRELATIONLISTPROTO,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ModelRelationListProto)
  })
_sym_db.RegisterMessage(ModelRelationListProto)

ModelVersionNameRequest = _reflection.GeneratedProtocolMessageType('ModelVersionNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODELVERSIONNAMEREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ModelVersionNameRequest)
  })
_sym_db.RegisterMessage(ModelVersionNameRequest)

RegisterModelVersionRelationRequest = _reflection.GeneratedProtocolMessageType('RegisterModelVersionRelationRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERMODELVERSIONRELATIONREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterModelVersionRelationRequest)
  })
_sym_db.RegisterMessage(RegisterModelVersionRelationRequest)

RegisterModelVersionRequest = _reflection.GeneratedProtocolMessageType('RegisterModelVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERMODELVERSIONREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterModelVersionRequest)
  })
_sym_db.RegisterMessage(RegisterModelVersionRequest)

ListModelVersionRelationRequest = _reflection.GeneratedProtocolMessageType('ListModelVersionRelationRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMODELVERSIONRELATIONREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListModelVersionRelationRequest)
  })
_sym_db.RegisterMessage(ListModelVersionRelationRequest)

ModelVersionRelationListProto = _reflection.GeneratedProtocolMessageType('ModelVersionRelationListProto', (_message.Message,), {
  'DESCRIPTOR' : _MODELVERSIONRELATIONLISTPROTO,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ModelVersionRelationListProto)
  })
_sym_db.RegisterMessage(ModelVersionRelationListProto)

ModelVersionListProto = _reflection.GeneratedProtocolMessageType('ModelVersionListProto', (_message.Message,), {
  'DESCRIPTOR' : _MODELVERSIONLISTPROTO,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ModelVersionListProto)
  })
_sym_db.RegisterMessage(ModelVersionListProto)

ModelNameRequest = _reflection.GeneratedProtocolMessageType('ModelNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODELNAMEREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ModelNameRequest)
  })
_sym_db.RegisterMessage(ModelNameRequest)

RegisterProjectRequest = _reflection.GeneratedProtocolMessageType('RegisterProjectRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERPROJECTREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterProjectRequest)
  })
_sym_db.RegisterMessage(RegisterProjectRequest)

UpdateProjectRequest = _reflection.GeneratedProtocolMessageType('UpdateProjectRequest', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEPROJECTREQUEST_PROPERTIESENTRY,
    '__module__' : 'metadata_service_pb2'
    # @@protoc_insertion_point(class_scope:ai_flow.UpdateProjectRequest.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _UPDATEPROJECTREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UpdateProjectRequest)
  })
_sym_db.RegisterMessage(UpdateProjectRequest)
_sym_db.RegisterMessage(UpdateProjectRequest.PropertiesEntry)

ProjectListProto = _reflection.GeneratedProtocolMessageType('ProjectListProto', (_message.Message,), {
  'DESCRIPTOR' : _PROJECTLISTPROTO,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ProjectListProto)
  })
_sym_db.RegisterMessage(ProjectListProto)

RegisterArtifactRequest = _reflection.GeneratedProtocolMessageType('RegisterArtifactRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERARTIFACTREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterArtifactRequest)
  })
_sym_db.RegisterMessage(RegisterArtifactRequest)

UpdateArtifactRequest = _reflection.GeneratedProtocolMessageType('UpdateArtifactRequest', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEARTIFACTREQUEST_PROPERTIESENTRY,
    '__module__' : 'metadata_service_pb2'
    # @@protoc_insertion_point(class_scope:ai_flow.UpdateArtifactRequest.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _UPDATEARTIFACTREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UpdateArtifactRequest)
  })
_sym_db.RegisterMessage(UpdateArtifactRequest)
_sym_db.RegisterMessage(UpdateArtifactRequest.PropertiesEntry)

ArtifactListProto = _reflection.GeneratedProtocolMessageType('ArtifactListProto', (_message.Message,), {
  'DESCRIPTOR' : _ARTIFACTLISTPROTO,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ArtifactListProto)
  })
_sym_db.RegisterMessage(ArtifactListProto)

RegisterWorkflowRequest = _reflection.GeneratedProtocolMessageType('RegisterWorkflowRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERWORKFLOWREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.RegisterWorkflowRequest)
  })
_sym_db.RegisterMessage(RegisterWorkflowRequest)

UpdateWorkflowRequest = _reflection.GeneratedProtocolMessageType('UpdateWorkflowRequest', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEWORKFLOWREQUEST_PROPERTIESENTRY,
    '__module__' : 'metadata_service_pb2'
    # @@protoc_insertion_point(class_scope:ai_flow.UpdateWorkflowRequest.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _UPDATEWORKFLOWREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UpdateWorkflowRequest)
  })
_sym_db.RegisterMessage(UpdateWorkflowRequest)
_sym_db.RegisterMessage(UpdateWorkflowRequest.PropertiesEntry)

WorkflowListProto = _reflection.GeneratedProtocolMessageType('WorkflowListProto', (_message.Message,), {
  'DESCRIPTOR' : _WORKFLOWLISTPROTO,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.WorkflowListProto)
  })
_sym_db.RegisterMessage(WorkflowListProto)

ListWorkflowsRequest = _reflection.GeneratedProtocolMessageType('ListWorkflowsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTWORKFLOWSREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListWorkflowsRequest)
  })
_sym_db.RegisterMessage(ListWorkflowsRequest)

WorkflowNameRequest = _reflection.GeneratedProtocolMessageType('WorkflowNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _WORKFLOWNAMEREQUEST,
  '__module__' : 'metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.WorkflowNameRequest)
  })
_sym_db.RegisterMessage(WorkflowNameRequest)


DESCRIPTOR._options = None
_UPDATEDATASETREQUEST_PROPERTIESENTRY._options = None
_UPDATEPROJECTREQUEST_PROPERTIESENTRY._options = None
_UPDATEARTIFACTREQUEST_PROPERTIESENTRY._options = None
_UPDATEWORKFLOWREQUEST_PROPERTIESENTRY._options = None

_METADATASERVICE = _descriptor.ServiceDescriptor(
  name='MetadataService',
  full_name='ai_flow.MetadataService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=3133,
  serialized_end=9414,
  methods=[
  _descriptor.MethodDescriptor(
    name='getDatasetById',
    full_name='ai_flow.MetadataService.getDatasetById',
    index=0,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002\'\022%/aiflow/metadata_store/dataset/get/id',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getDatasetByName',
    full_name='ai_flow.MetadataService.getDatasetByName',
    index=1,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002)\022\'/aiflow/metadata_store/dataset/get/name',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listDatasets',
    full_name='ai_flow.MetadataService.listDatasets',
    index=2,
    containing_service=None,
    input_type=_LISTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002%\022#/aiflow/metadata_store/dataset/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerDataset',
    full_name='ai_flow.MetadataService.registerDataset',
    index=3,
    containing_service=None,
    input_type=_REGISTERDATASETREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002(\"#/aiflow/metadata_store/dataset/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerDatasetWithCatalog',
    full_name='ai_flow.MetadataService.registerDatasetWithCatalog',
    index=4,
    containing_service=None,
    input_type=_REGISTERDATASETREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0025\"0/aiflow/metadata_store/dataset/save_with_catalog:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerDatasets',
    full_name='ai_flow.MetadataService.registerDatasets',
    index=5,
    containing_service=None,
    input_type=_REGISTERDATASETSREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002(\"#/aiflow/metadata_store/dataset/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateDataset',
    full_name='ai_flow.MetadataService.updateDataset',
    index=6,
    containing_service=None,
    input_type=_UPDATEDATASETREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002*\"%/aiflow/metadata_store/dataset/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteDatasetById',
    full_name='ai_flow.MetadataService.deleteDatasetById',
    index=7,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002-\"(/aiflow/metadata_store/dataset/delete/id:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteDatasetByName',
    full_name='ai_flow.MetadataService.deleteDatasetByName',
    index=8,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002/\"*/aiflow/metadata_store/dataset/delete/name:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getModelRelationById',
    full_name='ai_flow.MetadataService.getModelRelationById',
    index=9,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002.\022,/aiflow/metadata_store/model_relation/get/id',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getModelRelationByName',
    full_name='ai_flow.MetadataService.getModelRelationByName',
    index=10,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0020\022./aiflow/metadata_store/model_relation/get/name',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listModelRelation',
    full_name='ai_flow.MetadataService.listModelRelation',
    index=11,
    containing_service=None,
    input_type=_LISTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002,\022*/aiflow/metadata_store/model_relation/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerModelRelation',
    full_name='ai_flow.MetadataService.registerModelRelation',
    index=12,
    containing_service=None,
    input_type=_REGISTERMODELRELATIONREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002/\"*/aiflow/metadata_store/model_relation/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelRelationById',
    full_name='ai_flow.MetadataService.deleteModelRelationById',
    index=13,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0024\"//aiflow/metadata_store/model_relation/delete/id:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelRelationByName',
    full_name='ai_flow.MetadataService.deleteModelRelationByName',
    index=14,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0026\"1/aiflow/metadata_store/model_relation/delete/name:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getModelById',
    full_name='ai_flow.MetadataService.getModelById',
    index=15,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002%\022#/aiflow/metadata_store/model/get/id',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getModelByName',
    full_name='ai_flow.MetadataService.getModelByName',
    index=16,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002\'\022%/aiflow/metadata_store/model/get/name',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerModel',
    full_name='ai_flow.MetadataService.registerModel',
    index=17,
    containing_service=None,
    input_type=_REGISTERMODELREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/metadata_store/model/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelById',
    full_name='ai_flow.MetadataService.deleteModelById',
    index=18,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002+\"&/aiflow/metadata_store/model/delete/id:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelByName',
    full_name='ai_flow.MetadataService.deleteModelByName',
    index=19,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002-\"(/aiflow/metadata_store/model/delete/name:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionRelationByVersion',
    full_name='ai_flow.MetadataService.getModelVersionRelationByVersion',
    index=20,
    containing_service=None,
    input_type=_MODELVERSIONNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002:\0228/aiflow/metadata_store/modelVersion_relation/get/version',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listModelVersionRelation',
    full_name='ai_flow.MetadataService.listModelVersionRelation',
    index=21,
    containing_service=None,
    input_type=_LISTMODELVERSIONRELATIONREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0023\0221/aiflow/metadata_store/modelVersion_relation/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerModelVersionRelation',
    full_name='ai_flow.MetadataService.registerModelVersionRelation',
    index=22,
    containing_service=None,
    input_type=_REGISTERMODELVERSIONRELATIONREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0026\"1/aiflow/metadata_store/modelVersion_relation/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelVersionRelationByVersion',
    full_name='ai_flow.MetadataService.deleteModelVersionRelationByVersion',
    index=23,
    containing_service=None,
    input_type=_MODELVERSIONNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002@\";/aiflow/metadata_store/modelVersion_relation/delete/version:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionByVersion',
    full_name='ai_flow.MetadataService.getModelVersionByVersion',
    index=24,
    containing_service=None,
    input_type=_MODELVERSIONNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0021\022//aiflow/metadata_store/modelVersion/get/version',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerModelVersion',
    full_name='ai_flow.MetadataService.registerModelVersion',
    index=25,
    containing_service=None,
    input_type=_REGISTERMODELVERSIONREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002-\"(/aiflow/metadata_store/modelVersion/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelVersionByVersion',
    full_name='ai_flow.MetadataService.deleteModelVersionByVersion',
    index=26,
    containing_service=None,
    input_type=_MODELVERSIONNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0027\"2/aiflow/metadata_store/modelVersion/delete/version:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getDeployedModelVersion',
    full_name='ai_flow.MetadataService.getDeployedModelVersion',
    index=27,
    containing_service=None,
    input_type=_MODELNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0029\0227/aiflow/metadata_store/modelVersion/get_serving_version',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getLatestValidatedModelVersion',
    full_name='ai_flow.MetadataService.getLatestValidatedModelVersion',
    index=28,
    containing_service=None,
    input_type=_MODELNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002H\022F/aiflow/metadata_store/modelVersion/get_latest_validated_model_version',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getLatestGeneratedModelVersion',
    full_name='ai_flow.MetadataService.getLatestGeneratedModelVersion',
    index=29,
    containing_service=None,
    input_type=_MODELNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002H\022F/aiflow/metadata_store/modelVersion/get_latest_generated_model_version',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getProjectById',
    full_name='ai_flow.MetadataService.getProjectById',
    index=30,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002\'\022%/aiflow/metadata_store/project/get/id',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getProjectByName',
    full_name='ai_flow.MetadataService.getProjectByName',
    index=31,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002)\022\'/aiflow/metadata_store/project/get/name',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerProject',
    full_name='ai_flow.MetadataService.registerProject',
    index=32,
    containing_service=None,
    input_type=_REGISTERPROJECTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002(\"#/aiflow/metadata_store/project/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateProject',
    full_name='ai_flow.MetadataService.updateProject',
    index=33,
    containing_service=None,
    input_type=_UPDATEPROJECTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002*\"%/aiflow/metadata_store/project/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listProject',
    full_name='ai_flow.MetadataService.listProject',
    index=34,
    containing_service=None,
    input_type=_LISTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002%\022#/aiflow/metadata_store/project/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteProjectById',
    full_name='ai_flow.MetadataService.deleteProjectById',
    index=35,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002-\"(/aiflow/metadata_store/project/delete/id:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteProjectByName',
    full_name='ai_flow.MetadataService.deleteProjectByName',
    index=36,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002/\"*/aiflow/metadata_store/project/delete/name:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getArtifactById',
    full_name='ai_flow.MetadataService.getArtifactById',
    index=37,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002(\022&/aiflow/metadata_store/artifact/get/id',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getArtifactByName',
    full_name='ai_flow.MetadataService.getArtifactByName',
    index=38,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002*\022(/aiflow/metadata_store/artifact/get/name',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateArtifact',
    full_name='ai_flow.MetadataService.updateArtifact',
    index=39,
    containing_service=None,
    input_type=_UPDATEARTIFACTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002+\"&/aiflow/metadata_store/artifact/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerArtifact',
    full_name='ai_flow.MetadataService.registerArtifact',
    index=40,
    containing_service=None,
    input_type=_REGISTERARTIFACTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metadata_store/artifact/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listArtifact',
    full_name='ai_flow.MetadataService.listArtifact',
    index=41,
    containing_service=None,
    input_type=_LISTREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002&\022$/aiflow/metadata_store/artifact/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteArtifactById',
    full_name='ai_flow.MetadataService.deleteArtifactById',
    index=42,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002.\")/aiflow/metadata_store/artifact/delete/id:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteArtifactByName',
    full_name='ai_flow.MetadataService.deleteArtifactByName',
    index=43,
    containing_service=None,
    input_type=_NAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0020\"+/aiflow/metadata_store/artifact/delete/name:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerWorkflow',
    full_name='ai_flow.MetadataService.registerWorkflow',
    index=44,
    containing_service=None,
    input_type=_REGISTERWORKFLOWREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metadata_store/workflow/save:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateWorkflow',
    full_name='ai_flow.MetadataService.updateWorkflow',
    index=45,
    containing_service=None,
    input_type=_UPDATEWORKFLOWREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002+\"&/aiflow/metadata_store/workflow/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getWorkflowById',
    full_name='ai_flow.MetadataService.getWorkflowById',
    index=46,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002(\022&/aiflow/metadata_store/workflow/get/id',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getWorkflowByName',
    full_name='ai_flow.MetadataService.getWorkflowByName',
    index=47,
    containing_service=None,
    input_type=_WORKFLOWNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002*\022(/aiflow/metadata_store/workflow/get/name',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteWorkflowById',
    full_name='ai_flow.MetadataService.deleteWorkflowById',
    index=48,
    containing_service=None,
    input_type=_IDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002.\")/aiflow/metadata_store/workflow/delete/id:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteWorkflowByName',
    full_name='ai_flow.MetadataService.deleteWorkflowByName',
    index=49,
    containing_service=None,
    input_type=_WORKFLOWNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\0020\"+/aiflow/metadata_store/workflow/delete/name:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listWorkflows',
    full_name='ai_flow.MetadataService.listWorkflows',
    index=50,
    containing_service=None,
    input_type=_LISTWORKFLOWSREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002&\022$/aiflow/metadata_store/workflow/list',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_METADATASERVICE)

DESCRIPTOR.services_by_name['MetadataService'] = _METADATASERVICE

MetadataService = service_reflection.GeneratedServiceType('MetadataService', (_service.Service,), dict(
  DESCRIPTOR = _METADATASERVICE,
  __module__ = 'metadata_service_pb2'
  ))

MetadataService_Stub = service_reflection.GeneratedServiceStubType('MetadataService_Stub', (MetadataService,), dict(
  DESCRIPTOR = _METADATASERVICE,
  __module__ = 'metadata_service_pb2'
  ))


# @@protoc_insertion_point(module_scope)
