#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: model_center_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import message_pb2 as message__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='model_center_service.proto',
  package='ai_flow',
  syntax='proto3',
  serialized_options=b'\n\020com.aiflow.protoZ\010/ai_flow\210\001\001\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1amodel_center_service.proto\x12\x07\x61i_flow\x1a\rmessage.proto\x1a\x1cgoogle/api/annotations.proto\"W\n\x1c\x43reateRegisteredModelRequest\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x1d.ai_flow.RegisteredModelParam\"\x84\x01\n\x1cUpdateRegisteredModelRequest\x12+\n\nmodel_meta\x18\x01 \x01(\x0b\x32\x17.ai_flow.ModelMetaParam\x12\x37\n\x10registered_model\x18\x02 \x01(\x0b\x32\x1d.ai_flow.RegisteredModelParam\"K\n\x1c\x44\x65leteRegisteredModelRequest\x12+\n\nmodel_meta\x18\x01 \x01(\x0b\x32\x17.ai_flow.ModelMetaParam\"\x1d\n\x1bListRegisteredModelsRequest\"N\n\x1fGetRegisteredModelDetailRequest\x12+\n\nmodel_meta\x18\x01 \x01(\x0b\x32\x17.ai_flow.ModelMetaParam\"{\n\x19\x43reateModelVersionRequest\x12+\n\nmodel_meta\x18\x01 \x01(\x0b\x32\x17.ai_flow.ModelMetaParam\x12\x31\n\rmodel_version\x18\x02 \x01(\x0b\x32\x1a.ai_flow.ModelVersionParam\"{\n\x19UpdateModelVersionRequest\x12+\n\nmodel_meta\x18\x01 \x01(\x0b\x32\x17.ai_flow.ModelMetaParam\x12\x31\n\rmodel_version\x18\x02 \x01(\x0b\x32\x1a.ai_flow.ModelVersionParam\"H\n\x19\x44\x65leteModelVersionRequest\x12+\n\nmodel_meta\x18\x01 \x01(\x0b\x32\x17.ai_flow.ModelMetaParam\"K\n\x1cGetModelVersionDetailRequest\x12+\n\nmodel_meta\x18\x01 \x01(\x0b\x32\x17.ai_flow.ModelMetaParam2\xd4\t\n\x12ModelCenterService\x12\x88\x01\n\x15\x63reateRegisteredModel\x12%.ai_flow.CreateRegisteredModelRequest\x1a\x11.ai_flow.Response\"5\x82\xd3\xe4\x93\x02/\"*/aiflow/model-center/register-model/create:\x01*\x12\x88\x01\n\x15updateRegisteredModel\x12%.ai_flow.UpdateRegisteredModelRequest\x1a\x11.ai_flow.Response\"5\x82\xd3\xe4\x93\x02/2*/aiflow/model-center/register-model/update:\x01*\x12\x88\x01\n\x15\x64\x65leteRegisteredModel\x12%.ai_flow.DeleteRegisteredModelRequest\x1a\x11.ai_flow.Response\"5\x82\xd3\xe4\x93\x02/2*/aiflow/model-center/register-model/update:\x01*\x12\x81\x01\n\x14listRegisteredModels\x12$.ai_flow.ListRegisteredModelsRequest\x1a\x11.ai_flow.Response\"0\x82\xd3\xe4\x93\x02*\x12(/aiflow/model-center/register-model/list\x12\x88\x01\n\x18getRegisteredModelDetail\x12(.ai_flow.GetRegisteredModelDetailRequest\x1a\x11.ai_flow.Response\"/\x82\xd3\xe4\x93\x02)\x12\'/aiflow/model-center/register-model/get\x12\x81\x01\n\x12\x63reateModelVersion\x12\".ai_flow.CreateModelVersionRequest\x1a\x11.ai_flow.Response\"4\x82\xd3\xe4\x93\x02.\")/aiflow/model-center/model-version/create:\x01*\x12\x81\x01\n\x12updateModelVersion\x12\".ai_flow.UpdateModelVersionRequest\x1a\x11.ai_flow.Response\"4\x82\xd3\xe4\x93\x02.2)/aiflow/model-center/model-version/update:\x01*\x12\x81\x01\n\x12\x64\x65leteModelVersion\x12\".ai_flow.DeleteModelVersionRequest\x1a\x11.ai_flow.Response\"4\x82\xd3\xe4\x93\x02.2)/aiflow/model-center/model-version/delete:\x01*\x12\x81\x01\n\x15getModelVersionDetail\x12%.ai_flow.GetModelVersionDetailRequest\x1a\x11.ai_flow.Response\".\x82\xd3\xe4\x93\x02(\x12&/aiflow/model-center/model-version/getB\"\n\x10\x63om.aiflow.protoZ\x08/ai_flow\x88\x01\x01\x90\x01\x01\x62\x06proto3'
  ,
  dependencies=[message__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])




_CREATEREGISTEREDMODELREQUEST = _descriptor.Descriptor(
  name='CreateRegisteredModelRequest',
  full_name='ai_flow.CreateRegisteredModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='ai_flow.CreateRegisteredModelRequest.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=84,
  serialized_end=171,
)


_UPDATEREGISTEREDMODELREQUEST = _descriptor.Descriptor(
  name='UpdateRegisteredModelRequest',
  full_name='ai_flow.UpdateRegisteredModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_meta', full_name='ai_flow.UpdateRegisteredModelRequest.model_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='ai_flow.UpdateRegisteredModelRequest.registered_model', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=174,
  serialized_end=306,
)


_DELETEREGISTEREDMODELREQUEST = _descriptor.Descriptor(
  name='DeleteRegisteredModelRequest',
  full_name='ai_flow.DeleteRegisteredModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_meta', full_name='ai_flow.DeleteRegisteredModelRequest.model_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=308,
  serialized_end=383,
)


_LISTREGISTEREDMODELSREQUEST = _descriptor.Descriptor(
  name='ListRegisteredModelsRequest',
  full_name='ai_flow.ListRegisteredModelsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=385,
  serialized_end=414,
)


_GETREGISTEREDMODELDETAILREQUEST = _descriptor.Descriptor(
  name='GetRegisteredModelDetailRequest',
  full_name='ai_flow.GetRegisteredModelDetailRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_meta', full_name='ai_flow.GetRegisteredModelDetailRequest.model_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=416,
  serialized_end=494,
)


_CREATEMODELVERSIONREQUEST = _descriptor.Descriptor(
  name='CreateModelVersionRequest',
  full_name='ai_flow.CreateModelVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_meta', full_name='ai_flow.CreateModelVersionRequest.model_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_version', full_name='ai_flow.CreateModelVersionRequest.model_version', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=496,
  serialized_end=619,
)


_UPDATEMODELVERSIONREQUEST = _descriptor.Descriptor(
  name='UpdateModelVersionRequest',
  full_name='ai_flow.UpdateModelVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_meta', full_name='ai_flow.UpdateModelVersionRequest.model_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_version', full_name='ai_flow.UpdateModelVersionRequest.model_version', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=621,
  serialized_end=744,
)


_DELETEMODELVERSIONREQUEST = _descriptor.Descriptor(
  name='DeleteModelVersionRequest',
  full_name='ai_flow.DeleteModelVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_meta', full_name='ai_flow.DeleteModelVersionRequest.model_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=746,
  serialized_end=818,
)


_GETMODELVERSIONDETAILREQUEST = _descriptor.Descriptor(
  name='GetModelVersionDetailRequest',
  full_name='ai_flow.GetModelVersionDetailRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_meta', full_name='ai_flow.GetModelVersionDetailRequest.model_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=820,
  serialized_end=895,
)

_CREATEREGISTEREDMODELREQUEST.fields_by_name['registered_model'].message_type = message__pb2._REGISTEREDMODELPARAM
_UPDATEREGISTEREDMODELREQUEST.fields_by_name['model_meta'].message_type = message__pb2._MODELMETAPARAM
_UPDATEREGISTEREDMODELREQUEST.fields_by_name['registered_model'].message_type = message__pb2._REGISTEREDMODELPARAM
_DELETEREGISTEREDMODELREQUEST.fields_by_name['model_meta'].message_type = message__pb2._MODELMETAPARAM
_GETREGISTEREDMODELDETAILREQUEST.fields_by_name['model_meta'].message_type = message__pb2._MODELMETAPARAM
_CREATEMODELVERSIONREQUEST.fields_by_name['model_meta'].message_type = message__pb2._MODELMETAPARAM
_CREATEMODELVERSIONREQUEST.fields_by_name['model_version'].message_type = message__pb2._MODELVERSIONPARAM
_UPDATEMODELVERSIONREQUEST.fields_by_name['model_meta'].message_type = message__pb2._MODELMETAPARAM
_UPDATEMODELVERSIONREQUEST.fields_by_name['model_version'].message_type = message__pb2._MODELVERSIONPARAM
_DELETEMODELVERSIONREQUEST.fields_by_name['model_meta'].message_type = message__pb2._MODELMETAPARAM
_GETMODELVERSIONDETAILREQUEST.fields_by_name['model_meta'].message_type = message__pb2._MODELMETAPARAM
DESCRIPTOR.message_types_by_name['CreateRegisteredModelRequest'] = _CREATEREGISTEREDMODELREQUEST
DESCRIPTOR.message_types_by_name['UpdateRegisteredModelRequest'] = _UPDATEREGISTEREDMODELREQUEST
DESCRIPTOR.message_types_by_name['DeleteRegisteredModelRequest'] = _DELETEREGISTEREDMODELREQUEST
DESCRIPTOR.message_types_by_name['ListRegisteredModelsRequest'] = _LISTREGISTEREDMODELSREQUEST
DESCRIPTOR.message_types_by_name['GetRegisteredModelDetailRequest'] = _GETREGISTEREDMODELDETAILREQUEST
DESCRIPTOR.message_types_by_name['CreateModelVersionRequest'] = _CREATEMODELVERSIONREQUEST
DESCRIPTOR.message_types_by_name['UpdateModelVersionRequest'] = _UPDATEMODELVERSIONREQUEST
DESCRIPTOR.message_types_by_name['DeleteModelVersionRequest'] = _DELETEMODELVERSIONREQUEST
DESCRIPTOR.message_types_by_name['GetModelVersionDetailRequest'] = _GETMODELVERSIONDETAILREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateRegisteredModelRequest = _reflection.GeneratedProtocolMessageType('CreateRegisteredModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEREGISTEREDMODELREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.CreateRegisteredModelRequest)
  })
_sym_db.RegisterMessage(CreateRegisteredModelRequest)

UpdateRegisteredModelRequest = _reflection.GeneratedProtocolMessageType('UpdateRegisteredModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEREGISTEREDMODELREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UpdateRegisteredModelRequest)
  })
_sym_db.RegisterMessage(UpdateRegisteredModelRequest)

DeleteRegisteredModelRequest = _reflection.GeneratedProtocolMessageType('DeleteRegisteredModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEREGISTEREDMODELREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.DeleteRegisteredModelRequest)
  })
_sym_db.RegisterMessage(DeleteRegisteredModelRequest)

ListRegisteredModelsRequest = _reflection.GeneratedProtocolMessageType('ListRegisteredModelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTREGISTEREDMODELSREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListRegisteredModelsRequest)
  })
_sym_db.RegisterMessage(ListRegisteredModelsRequest)

GetRegisteredModelDetailRequest = _reflection.GeneratedProtocolMessageType('GetRegisteredModelDetailRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETREGISTEREDMODELDETAILREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.GetRegisteredModelDetailRequest)
  })
_sym_db.RegisterMessage(GetRegisteredModelDetailRequest)

CreateModelVersionRequest = _reflection.GeneratedProtocolMessageType('CreateModelVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEMODELVERSIONREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.CreateModelVersionRequest)
  })
_sym_db.RegisterMessage(CreateModelVersionRequest)

UpdateModelVersionRequest = _reflection.GeneratedProtocolMessageType('UpdateModelVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEMODELVERSIONREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UpdateModelVersionRequest)
  })
_sym_db.RegisterMessage(UpdateModelVersionRequest)

DeleteModelVersionRequest = _reflection.GeneratedProtocolMessageType('DeleteModelVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEMODELVERSIONREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.DeleteModelVersionRequest)
  })
_sym_db.RegisterMessage(DeleteModelVersionRequest)

GetModelVersionDetailRequest = _reflection.GeneratedProtocolMessageType('GetModelVersionDetailRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMODELVERSIONDETAILREQUEST,
  '__module__' : 'model_center_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.GetModelVersionDetailRequest)
  })
_sym_db.RegisterMessage(GetModelVersionDetailRequest)


DESCRIPTOR._options = None

_MODELCENTERSERVICE = _descriptor.ServiceDescriptor(
  name='ModelCenterService',
  full_name='ai_flow.ModelCenterService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=898,
  serialized_end=2134,
  methods=[
  _descriptor.MethodDescriptor(
    name='createRegisteredModel',
    full_name='ai_flow.ModelCenterService.createRegisteredModel',
    index=0,
    containing_service=None,
    input_type=_CREATEREGISTEREDMODELREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002/\"*/aiflow/model-center/register-model/create:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateRegisteredModel',
    full_name='ai_flow.ModelCenterService.updateRegisteredModel',
    index=1,
    containing_service=None,
    input_type=_UPDATEREGISTEREDMODELREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002/2*/aiflow/model-center/register-model/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteRegisteredModel',
    full_name='ai_flow.ModelCenterService.deleteRegisteredModel',
    index=2,
    containing_service=None,
    input_type=_DELETEREGISTEREDMODELREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002/2*/aiflow/model-center/register-model/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listRegisteredModels',
    full_name='ai_flow.ModelCenterService.listRegisteredModels',
    index=3,
    containing_service=None,
    input_type=_LISTREGISTEREDMODELSREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002*\022(/aiflow/model-center/register-model/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getRegisteredModelDetail',
    full_name='ai_flow.ModelCenterService.getRegisteredModelDetail',
    index=4,
    containing_service=None,
    input_type=_GETREGISTEREDMODELDETAILREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002)\022\'/aiflow/model-center/register-model/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='createModelVersion',
    full_name='ai_flow.ModelCenterService.createModelVersion',
    index=5,
    containing_service=None,
    input_type=_CREATEMODELVERSIONREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002.\")/aiflow/model-center/model-version/create:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateModelVersion',
    full_name='ai_flow.ModelCenterService.updateModelVersion',
    index=6,
    containing_service=None,
    input_type=_UPDATEMODELVERSIONREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002.2)/aiflow/model-center/model-version/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelVersion',
    full_name='ai_flow.ModelCenterService.deleteModelVersion',
    index=7,
    containing_service=None,
    input_type=_DELETEMODELVERSIONREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002.2)/aiflow/model-center/model-version/delete:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionDetail',
    full_name='ai_flow.ModelCenterService.getModelVersionDetail',
    index=8,
    containing_service=None,
    input_type=_GETMODELVERSIONDETAILREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002(\022&/aiflow/model-center/model-version/get',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MODELCENTERSERVICE)

DESCRIPTOR.services_by_name['ModelCenterService'] = _MODELCENTERSERVICE

ModelCenterService = service_reflection.GeneratedServiceType('ModelCenterService', (_service.Service,), dict(
  DESCRIPTOR = _MODELCENTERSERVICE,
  __module__ = 'model_center_service_pb2'
  ))

ModelCenterService_Stub = service_reflection.GeneratedServiceStubType('ModelCenterService_Stub', (ModelCenterService,), dict(
  DESCRIPTOR = _MODELCENTERSERVICE,
  __module__ = 'model_center_service_pb2'
  ))


# @@protoc_insertion_point(module_scope)
