#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scheduling_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import message_pb2 as message__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='scheduling_service.proto',
  package='ai_flow',
  syntax='proto3',
  serialized_options=b'\n\020com.aiflow.protoZ\010/ai_flow\210\001\001\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x18scheduling_service.proto\x12\x07\x61i_flow\x1a\rmessage.proto\x1a\x1cgoogle/api/annotations.proto\"\xc1\x01\n\x17ScheduleWorkflowRequest\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x15\n\rworkflow_name\x18\x02 \x01(\t\x12\x38\n\x04\x61rgs\x18\x03 \x03(\x0b\x32*.ai_flow.ScheduleWorkflowRequest.ArgsEntry\x12\x15\n\rworkflow_json\x18\x04 \x01(\t\x1a+\n\tArgsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xc2\x01\n\x18WorkflowExecutionRequest\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x14\n\x0c\x65xecution_id\x18\x02 \x01(\t\x12\x15\n\rworkflow_name\x18\x03 \x01(\t\x12\x39\n\x04\x61rgs\x18\x04 \x03(\x0b\x32+.ai_flow.WorkflowExecutionRequest.ArgsEntry\x1a+\n\tArgsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"O\n\x12ScheduleJobRequest\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\x10\n\x08job_name\x18\x02 \x01(\t\x12\x14\n\x0c\x65xecution_id\x18\x03 \x01(\t\"f\n\x14WorkflowInfoResponse\x12$\n\x06result\x18\x01 \x01(\x0b\x32\x14.ai_flow.ResultProto\x12(\n\x08workflow\x18\x02 \x01(\x0b\x32\x16.ai_flow.WorkflowProto\"o\n\x18ListWorkflowInfoResponse\x12$\n\x06result\x18\x01 \x01(\x0b\x32\x14.ai_flow.ResultProto\x12-\n\rworkflow_list\x18\x02 \x03(\x0b\x32\x16.ai_flow.WorkflowProto\"~\n\x19WorkflowExecutionResponse\x12$\n\x06result\x18\x01 \x01(\x0b\x32\x14.ai_flow.ResultProto\x12;\n\x12workflow_execution\x18\x02 \x01(\x0b\x32\x1f.ai_flow.WorkflowExecutionProto\"\x87\x01\n\x1dListWorkflowExecutionResponse\x12$\n\x06result\x18\x01 \x01(\x0b\x32\x14.ai_flow.ResultProto\x12@\n\x17workflow_execution_list\x18\x02 \x03(\x0b\x32\x1f.ai_flow.WorkflowExecutionProto\"W\n\x0fJobInfoResponse\x12$\n\x06result\x18\x01 \x01(\x0b\x32\x14.ai_flow.ResultProto\x12\x1e\n\x03job\x18\x02 \x01(\x0b\x32\x11.ai_flow.JobProto\"`\n\x13ListJobInfoResponse\x12$\n\x06result\x18\x01 \x01(\x0b\x32\x14.ai_flow.ResultProto\x12#\n\x08job_list\x18\x02 \x03(\x0b\x32\x11.ai_flow.JobProto2\xcb\x10\n\x11SchedulingService\x12\x80\x01\n\x0esubmitWorkflow\x12 .ai_flow.ScheduleWorkflowRequest\x1a\x1d.ai_flow.WorkflowInfoResponse\"-\x82\xd3\xe4\x93\x02\'\"\"/aiflow/scheduling/workflow/submit:\x01*\x12\x80\x01\n\x0e\x64\x65leteWorkflow\x12 .ai_flow.ScheduleWorkflowRequest\x1a\x1d.ai_flow.WorkflowInfoResponse\"-\x82\xd3\xe4\x93\x02\'\"\"/aiflow/scheduling/workflow/delete:\x01*\x12\x88\x01\n\x17pauseWorkflowScheduling\x12 .ai_flow.ScheduleWorkflowRequest\x1a\x1d.ai_flow.WorkflowInfoResponse\",\x82\xd3\xe4\x93\x02&\"!/aiflow/scheduling/workflow/pause:\x01*\x12\x8a\x01\n\x18resumeWorkflowScheduling\x12 .ai_flow.ScheduleWorkflowRequest\x1a\x1d.ai_flow.WorkflowInfoResponse\"-\x82\xd3\xe4\x93\x02\'\"\"/aiflow/scheduling/workflow/resume:\x01*\x12w\n\x0bgetWorkflow\x12 .ai_flow.ScheduleWorkflowRequest\x1a\x1d.ai_flow.WorkflowInfoResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/aiflow/scheduling/workflow/get\x12~\n\rlistWorkflows\x12 .ai_flow.ScheduleWorkflowRequest\x1a!.ai_flow.ListWorkflowInfoResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /aiflow/scheduling/workflow/list\x12\x9e\x01\n\x19startNewWorkflowExecution\x12!.ai_flow.WorkflowExecutionRequest\x1a\".ai_flow.WorkflowExecutionResponse\":\x82\xd3\xe4\x93\x02\x34\"//aiflow/scheduling/workflow_execution/start_new:\x01*\x12\xa1\x01\n\x19killAllWorkflowExecutions\x12!.ai_flow.WorkflowExecutionRequest\x1a&.ai_flow.ListWorkflowExecutionResponse\"9\x82\xd3\xe4\x93\x02\x33\"./aiflow/scheduling/workflow_execution/kill_all:\x01*\x12\x95\x01\n\x15killWorkflowExecution\x12!.ai_flow.WorkflowExecutionRequest\x1a\".ai_flow.WorkflowExecutionResponse\"5\x82\xd3\xe4\x93\x02/\"*/aiflow/scheduling/workflow_execution/kill:\x01*\x12\x90\x01\n\x14getWorkflowExecution\x12!.ai_flow.WorkflowExecutionRequest\x1a\".ai_flow.WorkflowExecutionResponse\"1\x82\xd3\xe4\x93\x02+\x12)/aiflow/scheduling/workflow_execution/get\x12\x97\x01\n\x16listWorkflowExecutions\x12!.ai_flow.WorkflowExecutionRequest\x1a&.ai_flow.ListWorkflowExecutionResponse\"2\x82\xd3\xe4\x93\x02,\x12*/aiflow/scheduling/workflow_execution/list\x12j\n\x08startJob\x12\x1b.ai_flow.ScheduleJobRequest\x1a\x18.ai_flow.JobInfoResponse\"\'\x82\xd3\xe4\x93\x02!\"\x1c/aiflow/scheduling/job/start:\x01*\x12h\n\x07stopJob\x12\x1b.ai_flow.ScheduleJobRequest\x1a\x18.ai_flow.JobInfoResponse\"&\x82\xd3\xe4\x93\x02 \"\x1b/aiflow/scheduling/job/stop:\x01*\x12n\n\nrestartJob\x12\x1b.ai_flow.ScheduleJobRequest\x1a\x18.ai_flow.JobInfoResponse\")\x82\xd3\xe4\x93\x02#\"\x1e/aiflow/scheduling/job/restart:\x01*\x12\x63\n\x06getJob\x12\x1b.ai_flow.ScheduleJobRequest\x1a\x18.ai_flow.JobInfoResponse\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/aiflow/scheduling/job/get\x12j\n\x08listJobs\x12\x1b.ai_flow.ScheduleJobRequest\x1a\x1c.ai_flow.ListJobInfoResponse\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/aiflow/scheduling/job/listB\"\n\x10\x63om.aiflow.protoZ\x08/ai_flow\x88\x01\x01\x90\x01\x01\x62\x06proto3'
  ,
  dependencies=[message__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])




_SCHEDULEWORKFLOWREQUEST_ARGSENTRY = _descriptor.Descriptor(
  name='ArgsEntry',
  full_name='ai_flow.ScheduleWorkflowRequest.ArgsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ai_flow.ScheduleWorkflowRequest.ArgsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ai_flow.ScheduleWorkflowRequest.ArgsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=276,
)

_SCHEDULEWORKFLOWREQUEST = _descriptor.Descriptor(
  name='ScheduleWorkflowRequest',
  full_name='ai_flow.ScheduleWorkflowRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='ai_flow.ScheduleWorkflowRequest.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_name', full_name='ai_flow.ScheduleWorkflowRequest.workflow_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='ai_flow.ScheduleWorkflowRequest.args', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_json', full_name='ai_flow.ScheduleWorkflowRequest.workflow_json', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SCHEDULEWORKFLOWREQUEST_ARGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=276,
)


_WORKFLOWEXECUTIONREQUEST_ARGSENTRY = _descriptor.Descriptor(
  name='ArgsEntry',
  full_name='ai_flow.WorkflowExecutionRequest.ArgsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ai_flow.WorkflowExecutionRequest.ArgsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ai_flow.WorkflowExecutionRequest.ArgsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=276,
)

_WORKFLOWEXECUTIONREQUEST = _descriptor.Descriptor(
  name='WorkflowExecutionRequest',
  full_name='ai_flow.WorkflowExecutionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='ai_flow.WorkflowExecutionRequest.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='execution_id', full_name='ai_flow.WorkflowExecutionRequest.execution_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_name', full_name='ai_flow.WorkflowExecutionRequest.workflow_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='ai_flow.WorkflowExecutionRequest.args', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_WORKFLOWEXECUTIONREQUEST_ARGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=279,
  serialized_end=473,
)


_SCHEDULEJOBREQUEST = _descriptor.Descriptor(
  name='ScheduleJobRequest',
  full_name='ai_flow.ScheduleJobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='ai_flow.ScheduleJobRequest.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_name', full_name='ai_flow.ScheduleJobRequest.job_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='execution_id', full_name='ai_flow.ScheduleJobRequest.execution_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=475,
  serialized_end=554,
)


_WORKFLOWINFORESPONSE = _descriptor.Descriptor(
  name='WorkflowInfoResponse',
  full_name='ai_flow.WorkflowInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='ai_flow.WorkflowInfoResponse.result', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow', full_name='ai_flow.WorkflowInfoResponse.workflow', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=556,
  serialized_end=658,
)


_LISTWORKFLOWINFORESPONSE = _descriptor.Descriptor(
  name='ListWorkflowInfoResponse',
  full_name='ai_flow.ListWorkflowInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='ai_flow.ListWorkflowInfoResponse.result', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_list', full_name='ai_flow.ListWorkflowInfoResponse.workflow_list', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=660,
  serialized_end=771,
)


_WORKFLOWEXECUTIONRESPONSE = _descriptor.Descriptor(
  name='WorkflowExecutionResponse',
  full_name='ai_flow.WorkflowExecutionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='ai_flow.WorkflowExecutionResponse.result', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_execution', full_name='ai_flow.WorkflowExecutionResponse.workflow_execution', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=773,
  serialized_end=899,
)


_LISTWORKFLOWEXECUTIONRESPONSE = _descriptor.Descriptor(
  name='ListWorkflowExecutionResponse',
  full_name='ai_flow.ListWorkflowExecutionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='ai_flow.ListWorkflowExecutionResponse.result', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workflow_execution_list', full_name='ai_flow.ListWorkflowExecutionResponse.workflow_execution_list', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=902,
  serialized_end=1037,
)


_JOBINFORESPONSE = _descriptor.Descriptor(
  name='JobInfoResponse',
  full_name='ai_flow.JobInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='ai_flow.JobInfoResponse.result', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job', full_name='ai_flow.JobInfoResponse.job', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1039,
  serialized_end=1126,
)


_LISTJOBINFORESPONSE = _descriptor.Descriptor(
  name='ListJobInfoResponse',
  full_name='ai_flow.ListJobInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='ai_flow.ListJobInfoResponse.result', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='job_list', full_name='ai_flow.ListJobInfoResponse.job_list', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1128,
  serialized_end=1224,
)

_SCHEDULEWORKFLOWREQUEST_ARGSENTRY.containing_type = _SCHEDULEWORKFLOWREQUEST
_SCHEDULEWORKFLOWREQUEST.fields_by_name['args'].message_type = _SCHEDULEWORKFLOWREQUEST_ARGSENTRY
_WORKFLOWEXECUTIONREQUEST_ARGSENTRY.containing_type = _WORKFLOWEXECUTIONREQUEST
_WORKFLOWEXECUTIONREQUEST.fields_by_name['args'].message_type = _WORKFLOWEXECUTIONREQUEST_ARGSENTRY
_WORKFLOWINFORESPONSE.fields_by_name['result'].message_type = message__pb2._RESULTPROTO
_WORKFLOWINFORESPONSE.fields_by_name['workflow'].message_type = message__pb2._WORKFLOWPROTO
_LISTWORKFLOWINFORESPONSE.fields_by_name['result'].message_type = message__pb2._RESULTPROTO
_LISTWORKFLOWINFORESPONSE.fields_by_name['workflow_list'].message_type = message__pb2._WORKFLOWPROTO
_WORKFLOWEXECUTIONRESPONSE.fields_by_name['result'].message_type = message__pb2._RESULTPROTO
_WORKFLOWEXECUTIONRESPONSE.fields_by_name['workflow_execution'].message_type = message__pb2._WORKFLOWEXECUTIONPROTO
_LISTWORKFLOWEXECUTIONRESPONSE.fields_by_name['result'].message_type = message__pb2._RESULTPROTO
_LISTWORKFLOWEXECUTIONRESPONSE.fields_by_name['workflow_execution_list'].message_type = message__pb2._WORKFLOWEXECUTIONPROTO
_JOBINFORESPONSE.fields_by_name['result'].message_type = message__pb2._RESULTPROTO
_JOBINFORESPONSE.fields_by_name['job'].message_type = message__pb2._JOBPROTO
_LISTJOBINFORESPONSE.fields_by_name['result'].message_type = message__pb2._RESULTPROTO
_LISTJOBINFORESPONSE.fields_by_name['job_list'].message_type = message__pb2._JOBPROTO
DESCRIPTOR.message_types_by_name['ScheduleWorkflowRequest'] = _SCHEDULEWORKFLOWREQUEST
DESCRIPTOR.message_types_by_name['WorkflowExecutionRequest'] = _WORKFLOWEXECUTIONREQUEST
DESCRIPTOR.message_types_by_name['ScheduleJobRequest'] = _SCHEDULEJOBREQUEST
DESCRIPTOR.message_types_by_name['WorkflowInfoResponse'] = _WORKFLOWINFORESPONSE
DESCRIPTOR.message_types_by_name['ListWorkflowInfoResponse'] = _LISTWORKFLOWINFORESPONSE
DESCRIPTOR.message_types_by_name['WorkflowExecutionResponse'] = _WORKFLOWEXECUTIONRESPONSE
DESCRIPTOR.message_types_by_name['ListWorkflowExecutionResponse'] = _LISTWORKFLOWEXECUTIONRESPONSE
DESCRIPTOR.message_types_by_name['JobInfoResponse'] = _JOBINFORESPONSE
DESCRIPTOR.message_types_by_name['ListJobInfoResponse'] = _LISTJOBINFORESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ScheduleWorkflowRequest = _reflection.GeneratedProtocolMessageType('ScheduleWorkflowRequest', (_message.Message,), {

  'ArgsEntry' : _reflection.GeneratedProtocolMessageType('ArgsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SCHEDULEWORKFLOWREQUEST_ARGSENTRY,
    '__module__' : 'scheduling_service_pb2'
    # @@protoc_insertion_point(class_scope:ai_flow.ScheduleWorkflowRequest.ArgsEntry)
    })
  ,
  'DESCRIPTOR' : _SCHEDULEWORKFLOWREQUEST,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ScheduleWorkflowRequest)
  })
_sym_db.RegisterMessage(ScheduleWorkflowRequest)
_sym_db.RegisterMessage(ScheduleWorkflowRequest.ArgsEntry)

WorkflowExecutionRequest = _reflection.GeneratedProtocolMessageType('WorkflowExecutionRequest', (_message.Message,), {

  'ArgsEntry' : _reflection.GeneratedProtocolMessageType('ArgsEntry', (_message.Message,), {
    'DESCRIPTOR' : _WORKFLOWEXECUTIONREQUEST_ARGSENTRY,
    '__module__' : 'scheduling_service_pb2'
    # @@protoc_insertion_point(class_scope:ai_flow.WorkflowExecutionRequest.ArgsEntry)
    })
  ,
  'DESCRIPTOR' : _WORKFLOWEXECUTIONREQUEST,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.WorkflowExecutionRequest)
  })
_sym_db.RegisterMessage(WorkflowExecutionRequest)
_sym_db.RegisterMessage(WorkflowExecutionRequest.ArgsEntry)

ScheduleJobRequest = _reflection.GeneratedProtocolMessageType('ScheduleJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _SCHEDULEJOBREQUEST,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ScheduleJobRequest)
  })
_sym_db.RegisterMessage(ScheduleJobRequest)

WorkflowInfoResponse = _reflection.GeneratedProtocolMessageType('WorkflowInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _WORKFLOWINFORESPONSE,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.WorkflowInfoResponse)
  })
_sym_db.RegisterMessage(WorkflowInfoResponse)

ListWorkflowInfoResponse = _reflection.GeneratedProtocolMessageType('ListWorkflowInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTWORKFLOWINFORESPONSE,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListWorkflowInfoResponse)
  })
_sym_db.RegisterMessage(ListWorkflowInfoResponse)

WorkflowExecutionResponse = _reflection.GeneratedProtocolMessageType('WorkflowExecutionResponse', (_message.Message,), {
  'DESCRIPTOR' : _WORKFLOWEXECUTIONRESPONSE,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.WorkflowExecutionResponse)
  })
_sym_db.RegisterMessage(WorkflowExecutionResponse)

ListWorkflowExecutionResponse = _reflection.GeneratedProtocolMessageType('ListWorkflowExecutionResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTWORKFLOWEXECUTIONRESPONSE,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListWorkflowExecutionResponse)
  })
_sym_db.RegisterMessage(ListWorkflowExecutionResponse)

JobInfoResponse = _reflection.GeneratedProtocolMessageType('JobInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _JOBINFORESPONSE,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.JobInfoResponse)
  })
_sym_db.RegisterMessage(JobInfoResponse)

ListJobInfoResponse = _reflection.GeneratedProtocolMessageType('ListJobInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTJOBINFORESPONSE,
  '__module__' : 'scheduling_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListJobInfoResponse)
  })
_sym_db.RegisterMessage(ListJobInfoResponse)


DESCRIPTOR._options = None
_SCHEDULEWORKFLOWREQUEST_ARGSENTRY._options = None
_WORKFLOWEXECUTIONREQUEST_ARGSENTRY._options = None

_SCHEDULINGSERVICE = _descriptor.ServiceDescriptor(
  name='SchedulingService',
  full_name='ai_flow.SchedulingService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1227,
  serialized_end=3350,
  methods=[
  _descriptor.MethodDescriptor(
    name='submitWorkflow',
    full_name='ai_flow.SchedulingService.submitWorkflow',
    index=0,
    containing_service=None,
    input_type=_SCHEDULEWORKFLOWREQUEST,
    output_type=_WORKFLOWINFORESPONSE,
    serialized_options=b'\202\323\344\223\002\'\"\"/aiflow/scheduling/workflow/submit:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteWorkflow',
    full_name='ai_flow.SchedulingService.deleteWorkflow',
    index=1,
    containing_service=None,
    input_type=_SCHEDULEWORKFLOWREQUEST,
    output_type=_WORKFLOWINFORESPONSE,
    serialized_options=b'\202\323\344\223\002\'\"\"/aiflow/scheduling/workflow/delete:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='pauseWorkflowScheduling',
    full_name='ai_flow.SchedulingService.pauseWorkflowScheduling',
    index=2,
    containing_service=None,
    input_type=_SCHEDULEWORKFLOWREQUEST,
    output_type=_WORKFLOWINFORESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/scheduling/workflow/pause:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='resumeWorkflowScheduling',
    full_name='ai_flow.SchedulingService.resumeWorkflowScheduling',
    index=3,
    containing_service=None,
    input_type=_SCHEDULEWORKFLOWREQUEST,
    output_type=_WORKFLOWINFORESPONSE,
    serialized_options=b'\202\323\344\223\002\'\"\"/aiflow/scheduling/workflow/resume:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getWorkflow',
    full_name='ai_flow.SchedulingService.getWorkflow',
    index=4,
    containing_service=None,
    input_type=_SCHEDULEWORKFLOWREQUEST,
    output_type=_WORKFLOWINFORESPONSE,
    serialized_options=b'\202\323\344\223\002!\022\037/aiflow/scheduling/workflow/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listWorkflows',
    full_name='ai_flow.SchedulingService.listWorkflows',
    index=5,
    containing_service=None,
    input_type=_SCHEDULEWORKFLOWREQUEST,
    output_type=_LISTWORKFLOWINFORESPONSE,
    serialized_options=b'\202\323\344\223\002\"\022 /aiflow/scheduling/workflow/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='startNewWorkflowExecution',
    full_name='ai_flow.SchedulingService.startNewWorkflowExecution',
    index=6,
    containing_service=None,
    input_type=_WORKFLOWEXECUTIONREQUEST,
    output_type=_WORKFLOWEXECUTIONRESPONSE,
    serialized_options=b'\202\323\344\223\0024\"//aiflow/scheduling/workflow_execution/start_new:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='killAllWorkflowExecutions',
    full_name='ai_flow.SchedulingService.killAllWorkflowExecutions',
    index=7,
    containing_service=None,
    input_type=_WORKFLOWEXECUTIONREQUEST,
    output_type=_LISTWORKFLOWEXECUTIONRESPONSE,
    serialized_options=b'\202\323\344\223\0023\"./aiflow/scheduling/workflow_execution/kill_all:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='killWorkflowExecution',
    full_name='ai_flow.SchedulingService.killWorkflowExecution',
    index=8,
    containing_service=None,
    input_type=_WORKFLOWEXECUTIONREQUEST,
    output_type=_WORKFLOWEXECUTIONRESPONSE,
    serialized_options=b'\202\323\344\223\002/\"*/aiflow/scheduling/workflow_execution/kill:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getWorkflowExecution',
    full_name='ai_flow.SchedulingService.getWorkflowExecution',
    index=9,
    containing_service=None,
    input_type=_WORKFLOWEXECUTIONREQUEST,
    output_type=_WORKFLOWEXECUTIONRESPONSE,
    serialized_options=b'\202\323\344\223\002+\022)/aiflow/scheduling/workflow_execution/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listWorkflowExecutions',
    full_name='ai_flow.SchedulingService.listWorkflowExecutions',
    index=10,
    containing_service=None,
    input_type=_WORKFLOWEXECUTIONREQUEST,
    output_type=_LISTWORKFLOWEXECUTIONRESPONSE,
    serialized_options=b'\202\323\344\223\002,\022*/aiflow/scheduling/workflow_execution/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='startJob',
    full_name='ai_flow.SchedulingService.startJob',
    index=11,
    containing_service=None,
    input_type=_SCHEDULEJOBREQUEST,
    output_type=_JOBINFORESPONSE,
    serialized_options=b'\202\323\344\223\002!\"\034/aiflow/scheduling/job/start:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='stopJob',
    full_name='ai_flow.SchedulingService.stopJob',
    index=12,
    containing_service=None,
    input_type=_SCHEDULEJOBREQUEST,
    output_type=_JOBINFORESPONSE,
    serialized_options=b'\202\323\344\223\002 \"\033/aiflow/scheduling/job/stop:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='restartJob',
    full_name='ai_flow.SchedulingService.restartJob',
    index=13,
    containing_service=None,
    input_type=_SCHEDULEJOBREQUEST,
    output_type=_JOBINFORESPONSE,
    serialized_options=b'\202\323\344\223\002#\"\036/aiflow/scheduling/job/restart:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getJob',
    full_name='ai_flow.SchedulingService.getJob',
    index=14,
    containing_service=None,
    input_type=_SCHEDULEJOBREQUEST,
    output_type=_JOBINFORESPONSE,
    serialized_options=b'\202\323\344\223\002\034\022\032/aiflow/scheduling/job/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listJobs',
    full_name='ai_flow.SchedulingService.listJobs',
    index=15,
    containing_service=None,
    input_type=_SCHEDULEJOBREQUEST,
    output_type=_LISTJOBINFORESPONSE,
    serialized_options=b'\202\323\344\223\002\035\022\033/aiflow/scheduling/job/list',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_SCHEDULINGSERVICE)

DESCRIPTOR.services_by_name['SchedulingService'] = _SCHEDULINGSERVICE

SchedulingService = service_reflection.GeneratedServiceType('SchedulingService', (_service.Service,), dict(
  DESCRIPTOR = _SCHEDULINGSERVICE,
  __module__ = 'scheduling_service_pb2'
  ))

SchedulingService_Stub = service_reflection.GeneratedServiceStubType('SchedulingService_Stub', (SchedulingService,), dict(
  DESCRIPTOR = _SCHEDULINGSERVICE,
  __module__ = 'scheduling_service_pb2'
  ))


# @@protoc_insertion_point(module_scope)
