/*!
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */body{position:relative;padding:70px 0 100px;min-height:100vh;background-color:#fff}nav{box-shadow:0 3px 3px #aaa;z-index:999}pre{overflow:auto;word-wrap:normal}pre,pre code{white-space:pre}pre code{overflow-wrap:normal}input,select{margin:0}div.container{width:98%;padding-left:15px;padding-right:15px}.navbar a{color:#51504f;font-weight:700}.navbar a .caret{color:#e2d2e2}.navbar-nav li.dropdown:focus-within>.dropdown-menu,.navbar-nav li.dropdown:hover>.dropdown-menu{display:block}.navbar-nav li.dropdown:hover>.dropdown-menu{z-index:1001}.navbar-brand{padding:0 15px;height:60px;display:inline-flex;align-items:center}.brand-logo{width:104px;height:40px;overflow:visible!important}@keyframes pinSpin{0%{transform:rotate(0) translateX(0)}to{transform:rotate(1turn) translateX(0)}}.navbar-brand:hover .brand-logo-pinwheel{transform-origin:17.66px 17.66px;animation:pinSpin 1.5s linear infinite}.navbar-user-icon{display:inline-flex;justify-content:center;align-items:center;vertical-align:middle;margin:-5px 0;width:30px;height:30px;border-radius:50%;color:#017cee;background-color:#c0defb}span.status_square{width:10px;height:10px;border:1px solid grey;display:inline-block;padding-left:0;cursor:pointer}div.squares{float:right;font-size:1}span.success{background-color:#1b8e49}span.up_for_retry{background-color:gold}span.up_for_reschedule{background-color:#00c7d4}span.started{background-color:#04d659}span.error{background-color:#e43921}span.queued{background-color:#cbcbcb}span.upstream_failed{background-color:orange}span.skipped{background-color:#f8cec8}.d3-tip{background:#000;color:#fff;border:solid;border-width:1px;border-radius:5px;padding:10px}input#execution_date{margin-bottom:0}table.highlighttable{width:100%;table-layout:fixed}div.linenodiv{padding-right:1px!important}.linenos{width:50px;border:0}div.linenodiv pre{padding-left:0;padding-right:4px;color:#8e8e8d;background-color:transparent;text-align:right}.wrap{white-space:pre-wrap}.code{font-family:monospace}#sql{border:1px solid #ccc;border-radius:5px}.ace_editor div{font:inherit!important}#ace_container{margin:10px 0}#sql_ace{visibility:hidden}table.dataframe{font-size:12px}table.dataframe tbody tr td{padding:2px}table.dataframe.dataTable thead>tr>th{padding:10px 20px 10px 10px}table.dataTable.dataframe thead .sorting{background:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABMAAAATCAQAAADYWf5HAAAAkElEQVQoz7XQMQ5AQBCF4dWQSJxC5wwax1Cq1e7BAdxD5SL+Tq/QCM1oNiJidwox0355mXnG/DrEtIQ6azioNZQxI0ykPhTQIwhCR+BmBYtlK7kLJYwWCcJA9M4qdrZrd8pPjZWPtOqdRQy320YSV17OatFC4euts6z39GYMKRPCTKY9UnPQ6P+GtMRfGtPnBCiqhAeJPmkqAAAAAElFTkSuQmCC) no-repeat 100%}table.dataTable.dataframe thead .sorting_desc{background:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABMAAAATCAYAAAByUDbMAAAAZUlEQVQ4y2NgGAWjYBSggaqGu5FA/BOIv2PBIPFEUgxjB+IdQPwfC94HxLykus4GiD+hGfQOiB3J8SojEE9EM2wuSJzcsFMG4ttQgx4DsRalkZENxL+AuJQaMcsGxBOAmGvopk8AVz1sLZgg0bsAAAAASUVORK5CYII=) no-repeat 100%}table.dataTable.dataframe thead .sorting_asc{background:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABMAAAATCAYAAAByUDbMAAAAZ0lEQVQ4y2NgGLKgquEuFxBPAGI2ahhWCsS/gDibUoO0gPgxEP8H4ttArEyuQYxAPBdqEAxPBImTY5gjEL9DM+wTENuQahAvEO9DMwiGdwAxOymGJQLxTyD+jgWDxCMZRsEoGAVoAADeemwtPcZI2wAAAABJRU5ErkJggg==) no-repeat 100%}.no-wrap{white-space:nowrap}div.form-inline{margin-bottom:5px}body div.panel{padding:0}.blur{filter:url(#blur-effect-1)}.legend-row{display:flex;align-items:center;justify-content:space-between}.legend-item{display:inline-flex;align-items:center;margin-right:4px;border-radius:4px;border:2px solid #cbcbcb;padding:0 3px;font-size:11px}.legend-item--no-border{border-color:transparent;padding:0}.legend-item:last-child{margin-right:0}.legend-item__swatch{display:inline-block;margin-right:3px;width:12px;height:12px;border-radius:2px;border:1px solid #8e8e8d;background:#fff}.legend-item__swatch--circle{border-radius:50%;width:14px;height:14px}.legend-item__swatch--no-border{border-color:transparent}#timezone-other,label[for=timezone-other]{margin:3px 20px;font-weight:400}#timezone-menu ul.typeahead.dropdown-menu{max-height:200px;overflow-y:auto;overflow-x:hidden}.accordion-toggle,.bootstrap-datetimepicker-widget .datepicker>div{display:block}.accordion-toggle.collapsed>.toggle-direction{transform:rotate(180deg)}.dag-import-error{white-space:pre}.hll{background-color:#ffc}.c{color:#408080;font-style:italic}.err{border:1px solid red}.k{color:green;font-weight:700}.o{color:#666}.cm{color:#408080;font-style:italic}.cp{color:#bc7a00}.c1,.cs{color:#408080;font-style:italic}.gd{color:#a00000}.ge{font-style:italic}.gr{color:red}.gh{color:navy;font-weight:700}.gi{color:#00a000}.go{color:#888}.gp{color:navy}.gp,.gs,.gu{font-weight:700}.gu{color:purple}.gt{color:#04d}.kc,.kd,.kn{color:green;font-weight:700}.kp,.kr{color:green}.kr{font-weight:700}.kt{color:#b00040}.m{color:#666}.s{color:#ba2121}.na{color:#7d9029}.nb{color:green}.nc{color:#00f;font-weight:700}.no{color:#800}.nd{color:#a2f}.ni{color:#999}.ne,.ni{font-weight:700}.ne{color:#d2413a}.nf{color:#00f}.nl{color:#a0a000}.nn{color:#00f}.nn,.nt{font-weight:700}.nt{color:green}.nv{color:#19177c}.ow{color:#a2f;font-weight:700}.w{color:#bbb}.mb,.mf,.mh,.mi,.mo{color:#666}.sb,.sc,.sd{color:#ba2121}.sd{font-style:italic}.s1,.s2{color:#ba2121}.footer{display:flex;align-items:center;justify-content:space-between;position:absolute;bottom:0;width:100%;height:60px;color:#8e8e8d;background-color:#f5f5f5}.dag-view-tools{margin:16px 0;padding:16px 0;background-color:#f0f0f0}.code-wrap{position:relative;margin-top:30px}.code-wrap-toggle{position:absolute;top:15px;right:15px}.search-input{position:relative}.search-input__input{padding-right:40px}.search-input__clear-btn{position:absolute;right:2px;top:2px;border:0}.btn-group{display:inline-flex}.panel-body .panel-group+div{overflow-x:auto}.task-instance-modal-column{margin-top:8px;overflow-x:auto}