#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
from typing import Dict, Text
from ai_flow.util.json_utils import Jsonable


class Properties(Dict[Text, Jsonable]):
    pass


EMPTY_PROPERTIES = Properties()


class ExecuteProperties(Jsonable):
    def __init__(self,
                 batch_properties: Properties = None,
                 stream_properties: Properties = None,
                 common_properties: Properties = None) -> None:
        super().__init__()
        if batch_properties is None:
            self.batch_properties = {}
        else:
            self.batch_properties = batch_properties

        if stream_properties is None:
            self.stream_properties = {}
        else:
            self.stream_properties = stream_properties

        if common_properties is None:
            self.common_properties = {}
        else:
            self.common_properties = common_properties
