/*!
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */.axis line,.axis path{fill:none;stroke:#000;shape-rendering:crispEdges}rect{stroke:#000;cursor:pointer}.d3-tip:after{box-sizing:border-box;display:inline;width:100%;line-height:1;color:rgba(0,0,0,.8);content:"\25BC";pointer-events:none;position:absolute;text-align:center}.d3-tip.n:after{margin:-1px 0 0;top:100%;left:0}