/*!
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

svg {
  overflow: hidden;
}

.node rect {
  stroke: #51504f;
  stroke-width: 1px;
  fill: #fff;
}

.edgeLabel rect {
  fill: #fff;
}

.edgePath {
  stroke: #51504f;
  stroke-width: 1px;
}

g.cluster rect {
  stroke: #fff;
  stroke-dasharray: 5;
  rx: 5;
  ry: 5;
  opacity: 0.5;
}

g.node rect {
  stroke: #fff;
  stroke-width: 3px;
  cursor: pointer;
}

g.node circle {
  stroke: #51504f;
  stroke-width: 3px;
  cursor: pointer;
}

g.node .label {
  font-size: inherit;
  font-weight: normal;
}

g.node text {
  cursor: pointer;
}

g.node.success rect {
  stroke: green;
}

g.node.up_for_retry rect {
  stroke: gold;
}

g.node.up_for_reschedule rect {
  stroke: turquoise;
}

g.node.sensing rect {
  stroke: lightseagreen;
}

g.node.queued rect {
  stroke: grey;
}

g.node.running rect {
  stroke: lime;
}

g.node.failed rect {
  stroke: red;
}

g.node.shutdown rect {
  stroke: blue;
}

g.node.upstream_failed rect {
  stroke: orange;
}

g.node.skipped rect {
  stroke: pink;
}

g.node.removed rect {
  stroke: black;
}

.svg-wrapper {
  border-radius: 4px;
  background-color: #f0f0f0;
  cursor: move;
}

.refresh-actions {
  float: right;
  display: inline-flex;
  align-items: center;
  top: 10px;
  right: 10px;
  position: relative;
}

.refresh-actions > .switch-label {
  margin: 0 10px 0 20px;
}

.loading-dots.refresh-loading {
  display: none;
}
