#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: metric_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from . import message_pb2 as message__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='metric_service.proto',
  package='ai_flow',
  syntax='proto3',
  serialized_options=b'\n\020com.aiflow.protoZ\010/ai_flow\210\001\001\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14metric_service.proto\x12\x07\x61i_flow\x1a\x1cgoogle/api/annotations.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\rmessage.proto\"(\n\x11MetricNameRequest\x12\x13\n\x0bmetric_name\x18\x01 \x01(\t\"\x1b\n\x0bUuidRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\x03\"B\n\x11MetricMetaRequest\x12-\n\x0bmetric_meta\x18\x01 \x01(\x0b\x32\x18.ai_flow.MetricMetaProto\"l\n\x12MetricMetaResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12-\n\x0bmetric_meta\x18\x03 \x01(\x0b\x32\x18.ai_flow.MetricMetaProto\"K\n\x14MetricSummaryRequest\x12\x33\n\x0emetric_summary\x18\x01 \x01(\x0b\x32\x1b.ai_flow.MetricSummaryProto\"u\n\x15MetricSummaryResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x33\n\x0emetric_summary\x18\x03 \x01(\x0b\x32\x1b.ai_flow.MetricSummaryProto\"i\n\x1dListDatasetMetricMetasRequest\x12\x14\n\x0c\x64\x61taset_name\x18\x01 \x01(\t\x12\x32\n\x0cproject_name\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\"e\n\x1bListModelMetricMetasRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x32\n\x0cproject_name\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\"r\n\x17ListMetricMetasResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12.\n\x0cmetric_metas\x18\x03 \x03(\x0b\x32\x18.ai_flow.MetricMetaProto\"\x96\x02\n\x1aListMetricSummariesRequest\x12\x31\n\x0bmetric_name\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x30\n\nmetric_key\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x33\n\rmodel_version\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12/\n\nstart_time\x18\x04 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\x12-\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\"}\n\x1bListMetricSummariesResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x35\n\x10metric_summaries\x18\x03 \x03(\x0b\x32\x1b.ai_flow.MetricSummaryProto2\x97\x0b\n\rMetricService\x12{\n\x12registerMetricMeta\x12\x1a.ai_flow.MetricMetaRequest\x1a\x1b.ai_flow.MetricMetaResponse\",\x82\xd3\xe4\x93\x02&\"!/aiflow/metric/metric_meta/create:\x01*\x12y\n\x10updateMetricMeta\x12\x1a.ai_flow.MetricMetaRequest\x1a\x1b.ai_flow.MetricMetaResponse\",\x82\xd3\xe4\x93\x02&\"!/aiflow/metric/metric_meta/update:\x01*\x12o\n\x10\x64\x65leteMetricMeta\x12\x1a.ai_flow.MetricNameRequest\x1a\x11.ai_flow.Response\",\x82\xd3\xe4\x93\x02&\"!/aiflow/metric/metric_meta/delete:\x01*\x12p\n\rgetMetricMeta\x12\x1a.ai_flow.MetricNameRequest\x1a\x1b.ai_flow.MetricMetaResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/aiflow/metric/metric_meta/get\x12\x93\x01\n\x16listDatasetMetricMetas\x12&.ai_flow.ListDatasetMetricMetasRequest\x1a .ai_flow.ListMetricMetasResponse\"/\x82\xd3\xe4\x93\x02)\x12\'/aiflow/metric/metric_meta/dataset/list\x12\x8d\x01\n\x14listModelMetricMetas\x12$.ai_flow.ListModelMetricMetasRequest\x1a .ai_flow.ListMetricMetasResponse\"-\x82\xd3\xe4\x93\x02\'\x12%/aiflow/metric/metric_meta/model/list\x12\x87\x01\n\x15registerMetricSummary\x12\x1d.ai_flow.MetricSummaryRequest\x1a\x1e.ai_flow.MetricSummaryResponse\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metric/metric_summary/create:\x01*\x12\x85\x01\n\x13updateMetricSummary\x12\x1d.ai_flow.MetricSummaryRequest\x1a\x1e.ai_flow.MetricSummaryResponse\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metric/metric_summary/update:\x01*\x12o\n\x13\x64\x65leteMetricSummary\x12\x14.ai_flow.UuidRequest\x1a\x11.ai_flow.Response\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metric/metric_summary/delete:\x01*\x12s\n\x10getMetricSummary\x12\x14.ai_flow.UuidRequest\x1a\x1e.ai_flow.MetricSummaryResponse\")\x82\xd3\xe4\x93\x02#\x12!/aiflow/metric/metric_summary/get\x12\x8c\x01\n\x13listMetricSummaries\x12#.ai_flow.ListMetricSummariesRequest\x1a$.ai_flow.ListMetricSummariesResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/aiflow/metric/metric_summary/listB\"\n\x10\x63om.aiflow.protoZ\x08/ai_flow\x88\x01\x01\x90\x01\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_api_dot_annotations__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,message__pb2.DESCRIPTOR,])




_METRICNAMEREQUEST = _descriptor.Descriptor(
  name='MetricNameRequest',
  full_name='ai_flow.MetricNameRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_name', full_name='ai_flow.MetricNameRequest.metric_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=110,
  serialized_end=150,
)


_UUIDREQUEST = _descriptor.Descriptor(
  name='UuidRequest',
  full_name='ai_flow.UuidRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='ai_flow.UuidRequest.uuid', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=152,
  serialized_end=179,
)


_METRICMETAREQUEST = _descriptor.Descriptor(
  name='MetricMetaRequest',
  full_name='ai_flow.MetricMetaRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_meta', full_name='ai_flow.MetricMetaRequest.metric_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=181,
  serialized_end=247,
)


_METRICMETARESPONSE = _descriptor.Descriptor(
  name='MetricMetaResponse',
  full_name='ai_flow.MetricMetaResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.MetricMetaResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.MetricMetaResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_meta', full_name='ai_flow.MetricMetaResponse.metric_meta', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=249,
  serialized_end=357,
)


_METRICSUMMARYREQUEST = _descriptor.Descriptor(
  name='MetricSummaryRequest',
  full_name='ai_flow.MetricSummaryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_summary', full_name='ai_flow.MetricSummaryRequest.metric_summary', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=359,
  serialized_end=434,
)


_METRICSUMMARYRESPONSE = _descriptor.Descriptor(
  name='MetricSummaryResponse',
  full_name='ai_flow.MetricSummaryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.MetricSummaryResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.MetricSummaryResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_summary', full_name='ai_flow.MetricSummaryResponse.metric_summary', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=436,
  serialized_end=553,
)


_LISTDATASETMETRICMETASREQUEST = _descriptor.Descriptor(
  name='ListDatasetMetricMetasRequest',
  full_name='ai_flow.ListDatasetMetricMetasRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset_name', full_name='ai_flow.ListDatasetMetricMetasRequest.dataset_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_name', full_name='ai_flow.ListDatasetMetricMetasRequest.project_name', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=555,
  serialized_end=660,
)


_LISTMODELMETRICMETASREQUEST = _descriptor.Descriptor(
  name='ListModelMetricMetasRequest',
  full_name='ai_flow.ListModelMetricMetasRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_name', full_name='ai_flow.ListModelMetricMetasRequest.model_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_name', full_name='ai_flow.ListModelMetricMetasRequest.project_name', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=662,
  serialized_end=763,
)


_LISTMETRICMETASRESPONSE = _descriptor.Descriptor(
  name='ListMetricMetasResponse',
  full_name='ai_flow.ListMetricMetasResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.ListMetricMetasResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.ListMetricMetasResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_metas', full_name='ai_flow.ListMetricMetasResponse.metric_metas', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=765,
  serialized_end=879,
)


_LISTMETRICSUMMARIESREQUEST = _descriptor.Descriptor(
  name='ListMetricSummariesRequest',
  full_name='ai_flow.ListMetricSummariesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_name', full_name='ai_flow.ListMetricSummariesRequest.metric_name', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_key', full_name='ai_flow.ListMetricSummariesRequest.metric_key', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_version', full_name='ai_flow.ListMetricSummariesRequest.model_version', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='ai_flow.ListMetricSummariesRequest.start_time', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_time', full_name='ai_flow.ListMetricSummariesRequest.end_time', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=882,
  serialized_end=1160,
)


_LISTMETRICSUMMARIESRESPONSE = _descriptor.Descriptor(
  name='ListMetricSummariesResponse',
  full_name='ai_flow.ListMetricSummariesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.ListMetricSummariesResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.ListMetricSummariesResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_summaries', full_name='ai_flow.ListMetricSummariesResponse.metric_summaries', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1162,
  serialized_end=1287,
)

_METRICMETAREQUEST.fields_by_name['metric_meta'].message_type = message__pb2._METRICMETAPROTO
_METRICMETARESPONSE.fields_by_name['metric_meta'].message_type = message__pb2._METRICMETAPROTO
_METRICSUMMARYREQUEST.fields_by_name['metric_summary'].message_type = message__pb2._METRICSUMMARYPROTO
_METRICSUMMARYRESPONSE.fields_by_name['metric_summary'].message_type = message__pb2._METRICSUMMARYPROTO
_LISTDATASETMETRICMETASREQUEST.fields_by_name['project_name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMODELMETRICMETASREQUEST.fields_by_name['project_name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICMETASRESPONSE.fields_by_name['metric_metas'].message_type = message__pb2._METRICMETAPROTO
_LISTMETRICSUMMARIESREQUEST.fields_by_name['metric_name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['metric_key'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['model_version'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['start_time'].message_type = google_dot_protobuf_dot_wrappers__pb2._INT64VALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['end_time'].message_type = google_dot_protobuf_dot_wrappers__pb2._INT64VALUE
_LISTMETRICSUMMARIESRESPONSE.fields_by_name['metric_summaries'].message_type = message__pb2._METRICSUMMARYPROTO
DESCRIPTOR.message_types_by_name['MetricNameRequest'] = _METRICNAMEREQUEST
DESCRIPTOR.message_types_by_name['UuidRequest'] = _UUIDREQUEST
DESCRIPTOR.message_types_by_name['MetricMetaRequest'] = _METRICMETAREQUEST
DESCRIPTOR.message_types_by_name['MetricMetaResponse'] = _METRICMETARESPONSE
DESCRIPTOR.message_types_by_name['MetricSummaryRequest'] = _METRICSUMMARYREQUEST
DESCRIPTOR.message_types_by_name['MetricSummaryResponse'] = _METRICSUMMARYRESPONSE
DESCRIPTOR.message_types_by_name['ListDatasetMetricMetasRequest'] = _LISTDATASETMETRICMETASREQUEST
DESCRIPTOR.message_types_by_name['ListModelMetricMetasRequest'] = _LISTMODELMETRICMETASREQUEST
DESCRIPTOR.message_types_by_name['ListMetricMetasResponse'] = _LISTMETRICMETASRESPONSE
DESCRIPTOR.message_types_by_name['ListMetricSummariesRequest'] = _LISTMETRICSUMMARIESREQUEST
DESCRIPTOR.message_types_by_name['ListMetricSummariesResponse'] = _LISTMETRICSUMMARIESRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MetricNameRequest = _reflection.GeneratedProtocolMessageType('MetricNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _METRICNAMEREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricNameRequest)
  })
_sym_db.RegisterMessage(MetricNameRequest)

UuidRequest = _reflection.GeneratedProtocolMessageType('UuidRequest', (_message.Message,), {
  'DESCRIPTOR' : _UUIDREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UuidRequest)
  })
_sym_db.RegisterMessage(UuidRequest)

MetricMetaRequest = _reflection.GeneratedProtocolMessageType('MetricMetaRequest', (_message.Message,), {
  'DESCRIPTOR' : _METRICMETAREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricMetaRequest)
  })
_sym_db.RegisterMessage(MetricMetaRequest)

MetricMetaResponse = _reflection.GeneratedProtocolMessageType('MetricMetaResponse', (_message.Message,), {
  'DESCRIPTOR' : _METRICMETARESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricMetaResponse)
  })
_sym_db.RegisterMessage(MetricMetaResponse)

MetricSummaryRequest = _reflection.GeneratedProtocolMessageType('MetricSummaryRequest', (_message.Message,), {
  'DESCRIPTOR' : _METRICSUMMARYREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricSummaryRequest)
  })
_sym_db.RegisterMessage(MetricSummaryRequest)

MetricSummaryResponse = _reflection.GeneratedProtocolMessageType('MetricSummaryResponse', (_message.Message,), {
  'DESCRIPTOR' : _METRICSUMMARYRESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricSummaryResponse)
  })
_sym_db.RegisterMessage(MetricSummaryResponse)

ListDatasetMetricMetasRequest = _reflection.GeneratedProtocolMessageType('ListDatasetMetricMetasRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTDATASETMETRICMETASREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListDatasetMetricMetasRequest)
  })
_sym_db.RegisterMessage(ListDatasetMetricMetasRequest)

ListModelMetricMetasRequest = _reflection.GeneratedProtocolMessageType('ListModelMetricMetasRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMODELMETRICMETASREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListModelMetricMetasRequest)
  })
_sym_db.RegisterMessage(ListModelMetricMetasRequest)

ListMetricMetasResponse = _reflection.GeneratedProtocolMessageType('ListMetricMetasResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETRICMETASRESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListMetricMetasResponse)
  })
_sym_db.RegisterMessage(ListMetricMetasResponse)

ListMetricSummariesRequest = _reflection.GeneratedProtocolMessageType('ListMetricSummariesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETRICSUMMARIESREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListMetricSummariesRequest)
  })
_sym_db.RegisterMessage(ListMetricSummariesRequest)

ListMetricSummariesResponse = _reflection.GeneratedProtocolMessageType('ListMetricSummariesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETRICSUMMARIESRESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListMetricSummariesResponse)
  })
_sym_db.RegisterMessage(ListMetricSummariesResponse)


DESCRIPTOR._options = None

_METRICSERVICE = _descriptor.ServiceDescriptor(
  name='MetricService',
  full_name='ai_flow.MetricService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1290,
  serialized_end=2721,
  methods=[
  _descriptor.MethodDescriptor(
    name='registerMetricMeta',
    full_name='ai_flow.MetricService.registerMetricMeta',
    index=0,
    containing_service=None,
    input_type=_METRICMETAREQUEST,
    output_type=_METRICMETARESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/metric/metric_meta/create:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateMetricMeta',
    full_name='ai_flow.MetricService.updateMetricMeta',
    index=1,
    containing_service=None,
    input_type=_METRICMETAREQUEST,
    output_type=_METRICMETARESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/metric/metric_meta/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteMetricMeta',
    full_name='ai_flow.MetricService.deleteMetricMeta',
    index=2,
    containing_service=None,
    input_type=_METRICNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/metric/metric_meta/delete:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getMetricMeta',
    full_name='ai_flow.MetricService.getMetricMeta',
    index=3,
    containing_service=None,
    input_type=_METRICNAMEREQUEST,
    output_type=_METRICMETARESPONSE,
    serialized_options=b'\202\323\344\223\002 \022\036/aiflow/metric/metric_meta/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listDatasetMetricMetas',
    full_name='ai_flow.MetricService.listDatasetMetricMetas',
    index=4,
    containing_service=None,
    input_type=_LISTDATASETMETRICMETASREQUEST,
    output_type=_LISTMETRICMETASRESPONSE,
    serialized_options=b'\202\323\344\223\002)\022\'/aiflow/metric/metric_meta/dataset/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listModelMetricMetas',
    full_name='ai_flow.MetricService.listModelMetricMetas',
    index=5,
    containing_service=None,
    input_type=_LISTMODELMETRICMETASREQUEST,
    output_type=_LISTMETRICMETASRESPONSE,
    serialized_options=b'\202\323\344\223\002\'\022%/aiflow/metric/metric_meta/model/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerMetricSummary',
    full_name='ai_flow.MetricService.registerMetricSummary',
    index=6,
    containing_service=None,
    input_type=_METRICSUMMARYREQUEST,
    output_type=_METRICSUMMARYRESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metric/metric_summary/create:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateMetricSummary',
    full_name='ai_flow.MetricService.updateMetricSummary',
    index=7,
    containing_service=None,
    input_type=_METRICSUMMARYREQUEST,
    output_type=_METRICSUMMARYRESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metric/metric_summary/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteMetricSummary',
    full_name='ai_flow.MetricService.deleteMetricSummary',
    index=8,
    containing_service=None,
    input_type=_UUIDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metric/metric_summary/delete:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getMetricSummary',
    full_name='ai_flow.MetricService.getMetricSummary',
    index=9,
    containing_service=None,
    input_type=_UUIDREQUEST,
    output_type=_METRICSUMMARYRESPONSE,
    serialized_options=b'\202\323\344\223\002#\022!/aiflow/metric/metric_summary/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listMetricSummaries',
    full_name='ai_flow.MetricService.listMetricSummaries',
    index=10,
    containing_service=None,
    input_type=_LISTMETRICSUMMARIESREQUEST,
    output_type=_LISTMETRICSUMMARIESRESPONSE,
    serialized_options=b'\202\323\344\223\002$\022\"/aiflow/metric/metric_summary/list',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_METRICSERVICE)

DESCRIPTOR.services_by_name['MetricService'] = _METRICSERVICE

MetricService = service_reflection.GeneratedServiceType('MetricService', (_service.Service,), dict(
  DESCRIPTOR = _METRICSERVICE,
  __module__ = 'metric_service_pb2'
  ))

MetricService_Stub = service_reflection.GeneratedServiceStubType('MetricService_Stub', (MetricService,), dict(
  DESCRIPTOR = _METRICSERVICE,
  __module__ = 'metric_service_pb2'
  ))


# @@protoc_insertion_point(module_scope)
