/*!
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

circle {
  stroke: #017cee;
  fill: #fff;
  stroke-width: 1.5px;
}

.node text {
  font: 11px sans-serif;
  pointer-events: none;
}

path.link {
  fill: none;
  stroke: #3182bd;
  stroke-width: 1.5px;
  opacity: 0.5;
}

rect.state {
  stroke: #51504f;
  shape-rendering: crispEdges;
  cursor: pointer;
}

rect.null,
rect.scheduled,
rect.undefined {
  fill: #fff;
}

rect.success {
  fill: green;
}

rect.running {
  fill: lime;
}

rect.failed {
  fill: red;
}

rect.queued {
  fill: gray;
}

rect.shutdown {
  fill: blue;
}

rect.upstream_failed {
  fill: orange;
}

rect.up_for_retry {
  fill: gold;
}

rect.up_for_reschedule {
  fill: turquoise;
}

rect.skipped {
  fill: pink;
}

rect.sensing {
  fill: lightseagreen;
}

.tooltip.in {
  opacity: 1;
  filter: alpha(opacity=100);
}

.axis path,
.axis line {
  fill: none;
  stroke: #51504f;
}

.axis text {
  font: 11px sans-serif;
}

g.parent circle {
  cursor: pointer;
}

g.node.collapsed circle {
  stroke: #444;
  stroke-width: 4px;
}

g.axis path {
  shape-rendering: crispEdges;
}

g.tick line {
  shape-rendering: crispEdges;
}
